#include "CodeOut.h"
#include "BasicTypes.h"
#include "PtgOutput.h"
#include "ptg_gen.h"
#include "vlLog.h"
#include "vlAccess.h"

#include <stdlib.h>


static int SendOutput(Tcl_Obj *obj);

static void PtgOutPrintString(PtgFilePtr file, char *string)
{
  do {
    
    char *nl = strchr(string, '\n');
    char *next;
    int len;
    if (nl)
    {
      len = (int)(nl - string);
      next = nl + 1;
    }
    else
    {
      len = strlen(string);
      next = (char *)0;
    }
    
    if (len >= 0)
    {
      if (file->destinfo)
        Tcl_AppendToObj((Tcl_Obj *)file->destinfo, string, len);
      else
        file->destinfo = Tcl_NewStringObj (string, len);
    }
    
    if (next && file->destinfo)
    {
      SendOutput ((Tcl_Obj *)file->destinfo);
      file->destinfo = (void *)0;
    }
    
    string = next;
  } while (string);
}

void PtgOutPrintChar(PtgFilePtr file, char c)
{
  if (c == '\n')
  {
    if (file->destinfo)
    {
      SendOutput ((Tcl_Obj *)file->destinfo);
      file->destinfo = (void *)0;
    }
  }
  else
  {
    if (file->destinfo)
      Tcl_AppendToObj((Tcl_Obj *)file->destinfo, &c, 1);
    else
      file->destinfo = Tcl_NewStringObj (&c, 1);
  }
}

static Tcl_Obj *out_window = (Tcl_Obj*)0;

static int SendOutput(Tcl_Obj *obj)
{
  Tcl_Obj *cmd[3], *cmdlst, *result;
  int ret;
  
  Tcl_IncrRefCount(obj);
  cmd[0] = Tcl_NewStringObj("::codeoutput::addline", -1);
  cmd[1] = out_window;
  cmd[2] = obj;
  
  cmdlst = Tcl_NewListObj(3, cmd);
  Tcl_IncrRefCount(cmdlst);
  ret = Tcl_GlobalEvalObj(TclInterp, cmdlst);
  Tcl_DecrRefCount(cmdlst);
  Tcl_DecrRefCount(obj);
  
  if (ret != TCL_OK)
  {
    vlError("Error in CodeOutputWindow.fw");
    return -1;
  }
  
  result = Tcl_GetObjResult(TclInterp);
  Tcl_GetIntFromObj((Tcl_Interp *)0, result, &ret);
  return ret;
}

PTGNode PTGOutWindow_i(PTGNode root, char *title, VLObjectPointer rootNode)
{
  VLList l;
  l.append("::codeoutput::opennew");
  l.append(title);
  l.append(rootNode);

  out_window = l.eval();
  Tcl_IncrRefCount(out_window);
  
  PtgFilePtr ptgfile = NewPtgFile((Tcl_Obj *)0, PtgOutPrintString);
  ptgfile->PrintChar = PtgOutPrintChar;
  PTGProcess(ptgfile, root);
  
  Tcl_DecrRefCount(out_window);
  out_window = (Tcl_Obj*)0;
  FreePtgFile(ptgfile);
  
  return root;
}


PTGNode PTGOutWindow(char *title, PTGNode root)
{
  root = PTGAddNL(root);
  VLString processorTarget = vlList("main::getParam","dest").eval();
  if (processorTarget.isNull()) {
        PTGOutWindow_i(root, title, vlGetRootNode());
  }
  else {
        PTGOutFile(processorTarget, root);
  }
  return root;  
}


