
#ifndef _VisContainerInterface2PtrLIST_H
#define _VisContainerInterface2PtrLIST_H

/* include header file defining VisContainerInterface2Ptr if VisBasicCode is set: */
#define EMPTYVisBasicCodeHDR
#ifndef EMPTYHDR
#include "VisBasicCode.h"
#endif
#undef EMPTYVisBasicCodeHDR

#include "VisContainerInterface2PtrFunctionTypes.h"

typedef struct _VisContainerInterface2PtrLE *VisContainerInterface2PtrList;
typedef VisContainerInterface2PtrList    *VisContainerInterface2PtrListPtr;

struct _VisContainerInterface2PtrLE {
VisContainerInterface2Ptr               head;
VisContainerInterface2PtrList   tail;
};

#define NULLVisContainerInterface2PtrList               ((VisContainerInterface2PtrList)0)
#define NullVisContainerInterface2PtrList()     ((VisContainerInterface2PtrList)0)
#define SingleVisContainerInterface2PtrList(e)  (ConsVisContainerInterface2PtrList((e),NULLVisContainerInterface2PtrList))

#if defined(__STDC__) || defined(__cplusplus)

extern void FinlVisContainerInterface2PtrList(void);
   /* Deallocates all VisContainerInterface2PtrLists. */
   /* Dynamic storage is allocated on first need */

extern VisContainerInterface2PtrList ConsVisContainerInterface2PtrList (VisContainerInterface2Ptr e, VisContainerInterface2PtrList l);
   /* Constructs a VisContainerInterface2PtrList of an element e and a given tail l.
    * e is the first element of the list. */

extern VisContainerInterface2Ptr HeadVisContainerInterface2PtrList (VisContainerInterface2PtrList l);
   /* Returns the first element of the list l.
    * The list l must not be empty. */

extern VisContainerInterface2PtrList TailVisContainerInterface2PtrList (VisContainerInterface2PtrList l);
   /* Returns the tail of the list l.
    * If l is empty, an empty list is returned. */

extern int LengthVisContainerInterface2PtrList (VisContainerInterface2PtrList l);
   /* Returns the number of elements in the list l. */

extern VisContainerInterface2Ptr IthElemVisContainerInterface2PtrList (VisContainerInterface2PtrList l, int i);
   /* Returns the i-th element of the List l. The head of l
    * is referred to as 1. If the value of
    * i is greater than the length of the list, an error
    * is reported and the program exits.
    */

extern VisContainerInterface2PtrList CopyVisContainerInterface2PtrList (VisContainerInterface2PtrList l, VisContainerInterface2PtrMapFct cp);
   /* Copies the list l. Elements are copied by calls of cp. */

extern VisContainerInterface2PtrList AppVisContainerInterface2PtrList (VisContainerInterface2PtrList l1, VisContainerInterface2PtrList l2);
   /* Concatenates two lists l1 and l2. The resulting list contains l2 at
    * the end of a copy of list l1. */

extern VisContainerInterface2PtrList AppElVisContainerInterface2PtrList (VisContainerInterface2PtrList l, VisContainerInterface2Ptr e);
   /* Appends an element e to the list l.
    * The list l is not copied, it is modified as a side-effect
    * of this function. */

extern void InsertAfterVisContainerInterface2PtrList (VisContainerInterface2PtrList l, VisContainerInterface2Ptr e);
   /* This function requires a non-empty list l. The element e is inserted
    * just after the first element of l. 
    * The list l is modified as a side-effect of this function. */

extern VisContainerInterface2PtrList OrderedInsertVisContainerInterface2PtrList (VisContainerInterface2PtrList l, VisContainerInterface2Ptr e, VisContainerInterface2PtrCmpFctType fcmp);
   /* Inserts the element e into the list l maintaining l in
    * ascending order with respect to the compare fcmp. 
    * The list l is modified as a side-effect of this function. */

extern VisContainerInterface2PtrListPtr RefEndConsVisContainerInterface2PtrList (VisContainerInterface2PtrListPtr addr, VisContainerInterface2Ptr e);
   /* Appends an element e to the end of a list given by its address addr.
    * The address where the next element may be appended is returned.
    * The list is modified as a side-effect of this function. */

extern VisContainerInterface2PtrListPtr RefEndAppVisContainerInterface2PtrList (VisContainerInterface2PtrListPtr addr, VisContainerInterface2PtrList l);
   /* Appends a list l to the end of a list given by its address addr.
    * The address where the next element may be appended is returned.
    * The list is modified as a side-effect of this function. */

extern int ElemInVisContainerInterface2PtrList (VisContainerInterface2Ptr e, VisContainerInterface2PtrList l, VisContainerInterface2PtrCmpFctType cmpfct);
   /* This function returns true (1) iff the element e is in the List l.
    * List elements are compared by the function cmpfct. */

extern VisContainerInterface2PtrList AddToSetVisContainerInterface2PtrList (VisContainerInterface2Ptr e, VisContainerInterface2PtrList l, VisContainerInterface2PtrCmpFctType cmpfct);
   /* A list is returned that has e as an element.
    * l is checked whether it already contains e using the compare
    * function cmpfct. */

extern VisContainerInterface2PtrList AddToOrderedSetVisContainerInterface2PtrList 
       (VisContainerInterface2Ptr e, VisContainerInterface2PtrList l, VisContainerInterface2PtrCmpFctType cmpfct);
   /* A list is returned that has e as an element.
    * l is checked whether it already contains e using the compare
    * function cmpfct. l is assumed to be ordered increasingly in the sense of
    * cmpfct. */

extern VisContainerInterface2PtrList MapVisContainerInterface2PtrList (VisContainerInterface2PtrList l, VisContainerInterface2PtrMapFct f);
   /* Returns a new VisContainerInterface2PtrList obtained by applying f to each element of l. */

extern int CompVisContainerInterface2PtrList (VisContainerInterface2PtrList l1, VisContainerInterface2PtrList l2, VisContainerInterface2PtrCmpFctType f);
   /* Compares the lists l1 and l2 lexicographically by applying f
    * to the corresponding elements. */

extern VisContainerInterface2Ptr SumVisContainerInterface2PtrList (VisContainerInterface2PtrList l, VisContainerInterface2PtrSumFct f, VisContainerInterface2Ptr a);
   /* Applies the binary function f to the elements of the List:
    *   f( f(... f(a, e1), e2, ...), en)
    * If l is empty a is returned. */

#else

extern void FinlVisContainerInterface2PtrList ();
extern VisContainerInterface2PtrList ConsVisContainerInterface2PtrList ();
extern VisContainerInterface2Ptr HeadVisContainerInterface2PtrList ();
extern VisContainerInterface2PtrList TailVisContainerInterface2PtrList ();
extern int LengthVisContainerInterface2PtrList ();
extern VisContainerInterface2Ptr IthElemVisContainerInterface2PtrList ();
extern VisContainerInterface2PtrList CopyVisContainerInterface2PtrList ();
extern VisContainerInterface2PtrList AppVisContainerInterface2PtrList ();
extern VisContainerInterface2PtrList AppElVisContainerInterface2PtrList ();
extern void InsertAfterVisContainerInterface2PtrList ();
extern VisContainerInterface2PtrList OrderedInsertVisContainerInterface2PtrList ();
extern VisContainerInterface2PtrListPtr RefEndConsVisContainerInterface2PtrList ();
extern VisContainerInterface2PtrListPtr RefEndAppVisContainerInterface2PtrList ();
extern int ElemInVisContainerInterface2PtrList ();
extern VisContainerInterface2PtrList AddToSetVisContainerInterface2PtrList ();
extern VisContainerInterface2PtrList MapVisContainerInterface2PtrList ();
extern int CompVisContainerInterface2PtrList ();
extern VisContainerInterface2Ptr SumVisContainerInterface2PtrList ();

#endif

#ifdef MONITOR
#define DAPTO_RESULTVisContainerInterface2PtrList(l) DAPTO_RESULT_PTR(l)
#define DAPTO_ARGVisContainerInterface2PtrList(l)    DAPTO_ARG_PTR (l, VisContainerInterface2PtrList)
#endif

#endif
