#include <stdio.h>
#include <stdlib.h>

#include "ptg_gen.h"
#include "obstack.h"

/* -------------------------------------------------------- */
/*                      Memory Management                   */
/* -------------------------------------------------------- */

static Obstack _PTGObstack;
static void *_PTGFirstObj = NULL;

static void _PTGInit()
{
	if (_PTGFirstObj)
		return;
	obstack_init(&_PTGObstack);
	_PTGFirstObj = obstack_alloc(&_PTGObstack, 0);
}

void PTGFree()
{
	obstack_free(&_PTGObstack, _PTGFirstObj);
	_PTGFirstObj = obstack_alloc(&_PTGObstack, 0);
}


#if defined(__STDC__) || defined(__cplusplus)
static void *MALLOC(int size)
#else
static void *MALLOC(size) int size;
#endif
{
	if (! _PTGFirstObj)
		_PTGInit();
	return (obstack_alloc(&_PTGObstack, size));
}

/* -------------------------------------------------------- */
/*                       Output-functions                   */
/* -------------------------------------------------------- */

static PTG_OUTPUT_FILE f;
static char buffer[40];

#if PTG_OUTPUT_DEFAULT
#if defined(__STDC__) || defined(__cplusplus)
PTGNode PTGOut (PTGNode r)
#else
PTGNode PTGOut (r)
	PTGNode r;
#endif
{
	f = stdout;
	if (r) {
		(* (r->_print)) (r);
	}
	return (r);
}

#if defined(__STDC__) || defined(__cplusplus)
PTGNode PTGOutFile (char *fn, PTGNode r)
#else
PTGNode PTGOutFile (fn, r)
	char * fn; PTGNode r;
#endif
{
	if ((f = fopen(fn, "w"))  == (FILE *)NULL)
	{
		fprintf(stderr, "ERROR: PTGOutFile: output file '%s' can't be opened.\n",fn);
		exit(1);
	}
	if (r)
		(* (r->_print)) (r);
	fclose(f);
	return (r);
}

#if defined(__STDC__) || defined(__cplusplus)
PTGNode PTGOutFPtr(FILE *fptr, PTGNode r)
#else
PTGNode PTGOutFPtr(fptr, r)
	FILE *fptr; PTGNode r;
#endif
{
	if ((f = fptr) == (FILE *)NULL)
	{
		fprintf(stderr, "ERROR: PTGOutFPtr: output file not open.\n");
		exit(1);
	}
	if (r)
		(* (r->_print)) (r);
	return (r);
}

#endif

#if defined(__STDC__) || defined(__cplusplus)
PTGNode PTGProcess(PTG_OUTPUT_FILE file, PTGNode r)
#else
PTGNode PTGProcess(file, r)
	PTG_OUTPUT_FILE file; PTGNode r;
#endif
{
	f = file;
	if (r)
		(* (r->_print)) (r);
	return (r);
}
/* -------------------------------------------------------- */
/*                            PTGNULL                       */
/* -------------------------------------------------------- */

/* Define PTGNULL as a PTGNode that prints nothing. */

#if defined(__STDC__) || defined(__cplusplus)
static void _PrPTGNULL(_PPTG0 n)
#else
static void _PrPTGNULL(n)
_PPTG0 n;
#endif
{(void)n; /* function printing nothing */}

struct _SPTG0   _PTGNULL = { (_PTGProc) _PrPTGNULL };

/* -------------------------------------------------------- */
/*          Node-Construction and Print-functions           */
/* -------------------------------------------------------- */


/* Implementation of Pattern AddNL */

typedef struct _SPTGAddNL{
	_PTGProc _print;
	PTGNode p1;
} * _PPTGAddNL;

#ifdef PROTO_OK
static void _PrPTGAddNL(_PPTGAddNL n)
#else
static void _PrPTGAddNL(n)
	_PPTGAddNL n;
#endif
{
	n->p1->_print(n->p1);
	PTG_OUTPUT_STRING(f, "\n");
}

#ifdef PROTO_OK
PTGNode PTGAddNL(PTGNode p1)
#else
PTGNode PTGAddNL(p1)
PTGNode p1;
#endif
{
	_PPTGAddNL n;

	n = (_PPTGAddNL)MALLOC(sizeof(struct _SPTGAddNL));
	n->_print = (_PTGProc)_PrPTGAddNL;
	n->p1 = p1;
	return (PTGNode)n;
}

/* Implementation of Pattern Id */

typedef struct _SPTGId{
	_PTGProc _print;
	int p1;
} * _PPTGId;

#ifdef PROTO_OK
static void _PrPTGId(_PPTGId n)
#else
static void _PrPTGId(n)
	_PPTGId n;
#endif
{
	PtgOutId(f, n->p1);
}

#ifdef PROTO_OK
PTGNode PTGId(int p1)
#else
PTGNode PTGId(p1)
int p1;
#endif
{
	_PPTGId n;

	n = (_PPTGId)MALLOC(sizeof(struct _SPTGId));
	n->_print = (_PTGProc)_PrPTGId;
	n->p1 = p1;
	return (PTGNode)n;
}

/* Implementation of Pattern AsIs */

typedef struct _SPTGAsIs{
	_PTGProc _print;
	char * p1;
} * _PPTGAsIs;

#ifdef PROTO_OK
static void _PrPTGAsIs(_PPTGAsIs n)
#else
static void _PrPTGAsIs(n)
	_PPTGAsIs n;
#endif
{
	PTG_OUTPUT_STRING(f, n->p1);
}

#ifdef PROTO_OK
PTGNode PTGAsIs(char * p1)
#else
PTGNode PTGAsIs(p1)
char * p1;
#endif
{
	_PPTGAsIs n;

	n = (_PPTGAsIs)MALLOC(sizeof(struct _SPTGAsIs));
	n->_print = (_PTGProc)_PrPTGAsIs;
	n->p1 = p1;
	return (PTGNode)n;
}

/* Implementation of Pattern Numb */

typedef struct _SPTGNumb{
	_PTGProc _print;
	int p1;
} * _PPTGNumb;

#ifdef PROTO_OK
static void _PrPTGNumb(_PPTGNumb n)
#else
static void _PrPTGNumb(n)
	_PPTGNumb n;
#endif
{
	PTG_OUTPUT_INT(f, n->p1);
}

#ifdef PROTO_OK
PTGNode PTGNumb(int p1)
#else
PTGNode PTGNumb(p1)
int p1;
#endif
{
	_PPTGNumb n;

	n = (_PPTGNumb)MALLOC(sizeof(struct _SPTGNumb));
	n->_print = (_PTGProc)_PrPTGNumb;
	n->p1 = p1;
	return (PTGNode)n;
}

/* Implementation of Pattern CString */

typedef struct _SPTGCString{
	_PTGProc _print;
	char * p1;
} * _PPTGCString;

#ifdef PROTO_OK
static void _PrPTGCString(_PPTGCString n)
#else
static void _PrPTGCString(n)
	_PPTGCString n;
#endif
{
	CPtgOutstr(f, n->p1);
}

#ifdef PROTO_OK
PTGNode PTGCString(char * p1)
#else
PTGNode PTGCString(p1)
char * p1;
#endif
{
	_PPTGCString n;

	n = (_PPTGCString)MALLOC(sizeof(struct _SPTGCString));
	n->_print = (_PTGProc)_PrPTGCString;
	n->p1 = p1;
	return (PTGNode)n;
}

/* Implementation of Pattern CChar */

typedef struct _SPTGCChar{
	_PTGProc _print;
	int p1;
} * _PPTGCChar;

#ifdef PROTO_OK
static void _PrPTGCChar(_PPTGCChar n)
#else
static void _PrPTGCChar(n)
	_PPTGCChar n;
#endif
{
	CPtgOutchar(f, n->p1);
}

#ifdef PROTO_OK
PTGNode PTGCChar(int p1)
#else
PTGNode PTGCChar(p1)
int p1;
#endif
{
	_PPTGCChar n;

	n = (_PPTGCChar)MALLOC(sizeof(struct _SPTGCChar));
	n->_print = (_PTGProc)_PrPTGCChar;
	n->p1 = p1;
	return (PTGNode)n;
}

/* Implementation of Pattern PString */

typedef struct _SPTGPString{
	_PTGProc _print;
	char * p1;
} * _PPTGPString;

#ifdef PROTO_OK
static void _PrPTGPString(_PPTGPString n)
#else
static void _PrPTGPString(n)
	_PPTGPString n;
#endif
{
	PPtgOutstr(f, n->p1);
}

#ifdef PROTO_OK
PTGNode PTGPString(char * p1)
#else
PTGNode PTGPString(p1)
char * p1;
#endif
{
	_PPTGPString n;

	n = (_PPTGPString)MALLOC(sizeof(struct _SPTGPString));
	n->_print = (_PTGProc)_PrPTGPString;
	n->p1 = p1;
	return (PTGNode)n;
}

/* Implementation of Pattern Seq */

typedef struct _SPTGSeq{
	_PTGProc _print;
	PTGNode p1;
	PTGNode p2;
} * _PPTGSeq;

#ifdef PROTO_OK
static void _PrPTGSeq(_PPTGSeq n)
#else
static void _PrPTGSeq(n)
	_PPTGSeq n;
#endif
{
	n->p1->_print(n->p1);
	n->p2->_print(n->p2);
}

#ifdef PROTO_OK
PTGNode PTGSeq(PTGNode p1, PTGNode p2)
#else
PTGNode PTGSeq(p1, p2)
PTGNode p1;
PTGNode p2;
#endif
{
	_PPTGSeq n;

	n = (_PPTGSeq)MALLOC(sizeof(struct _SPTGSeq));
	if(p1 == PTGNULL && p2 == PTGNULL)
		return PTGNULL;

	n->_print = (_PTGProc)_PrPTGSeq;
	n->p1 = p1;
	n->p2 = p2;
	return (PTGNode)n;
}

/* Implementation of Pattern CommaSeq */

typedef struct _SPTGCommaSeq{
	_PTGProc _print;
	PTGNode p1;
	PTGNode p2;
} * _PPTGCommaSeq;

#ifdef PROTO_OK
static void _PrPTGCommaSeq(_PPTGCommaSeq n)
#else
static void _PrPTGCommaSeq(n)
	_PPTGCommaSeq n;
#endif
{
	n->p1->_print(n->p1);
	if (n->p1 != PTGNULL && n->p2 != PTGNULL)
	{
		PTG_OUTPUT_STRING(f, ", ");
	}
	n->p2->_print(n->p2);
}

#ifdef PROTO_OK
PTGNode PTGCommaSeq(PTGNode p1, PTGNode p2)
#else
PTGNode PTGCommaSeq(p1, p2)
PTGNode p1;
PTGNode p2;
#endif
{
	_PPTGCommaSeq n;

	n = (_PPTGCommaSeq)MALLOC(sizeof(struct _SPTGCommaSeq));
	if(p1 == PTGNULL && p2 == PTGNULL)
		return PTGNULL;

	n->_print = (_PTGProc)_PrPTGCommaSeq;
	n->p1 = p1;
	n->p2 = p2;
	return (PTGNode)n;
}

/* No Implementation of Pattern Eol needed */
/* See AddNL for implementation */

/* Implementation of Pattern WebApplication */

typedef struct _SPTGWebApplication{
	_PTGProc _print;
} * _PPTGWebApplication;

#ifdef PROTO_OK
static void _PrPTGWebApplication(_PPTGWebApplication n)
#else
static void _PrPTGWebApplication(n)
	_PPTGWebApplication n;
#endif
{
	PTG_OUTPUT_STRING(f, "<!-- TODO: WebApplication -->");
	IndentNewLine(f);
}

static struct _SPTGWebApplication _sptgWebApplication = { (_PTGProc) _PrPTGWebApplication };

#ifdef PROTO_OK
PTGNode PTGWebApplication(void)
#else
PTGNode PTGWebApplication()
#endif
{
	return (PTGNode)(&_sptgWebApplication);
}

/* Implementation of Pattern WebPage */

typedef struct _SPTGWebPage{
	_PTGProc _print;
	PTGNode p1;
	PTGNode p2;
	PTGNode p3;
	PTGNode p4;
	PTGNode p5;
	PTGNode p6;
} * _PPTGWebPage;

#ifdef PROTO_OK
static void _PrPTGWebPage(_PPTGWebPage n)
#else
static void _PrPTGWebPage(n)
	_PPTGWebPage n;
#endif
{
	PTG_OUTPUT_STRING(f, "<?php");
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "/* Starten des OutputBuffering */");
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "ob_start();");
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "session_start();");
	IndentNewLine(f);
	n->p5->_print(n->p5);
	PTG_OUTPUT_STRING(f, "echo '<?xml version=\"1.0\" encoding=\"utf-8\"?>', \"\n\"; ?>");
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "<!DOCTYPE html PUBLIC \"-//W3C//DTD XHTML 1.1//EN\"");
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "\"http://www.w3.org/TR/xhtml11/DTD/xhtml11.dtd\">");
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "<html xmlns=\"http://www.w3.org/1999/xhtml\">");
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "<head>");
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "<meta http-equiv='content-type' content='application/xhtml+xml; charset=utf-8'> </meta>");
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "<title>");
	n->p1->_print(n->p1);
	PTG_OUTPUT_STRING(f, "</title>");
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "<link rel=\"stylesheet\" media=\"all\" type=\"text/css\" href=\"");
	n->p2->_print(n->p2);
	PTG_OUTPUT_STRING(f, "\"> </link>");
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "<?php Include('");
	n->p3->_print(n->p3);
	PTG_OUTPUT_STRING(f, "');?>");
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "<?php Include('connect.php');?>");
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "<?php Include('globals.php');?>");
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "<?php");
	IndentNewLine(f);
	n->p6->_print(n->p6);
	IndentDecr(f);
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "?>");
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "</head>");
	IndentNewLine(f);
	n->p4->_print(n->p4);
	PTG_OUTPUT_STRING(f, "</html>");
	IndentNewLine(f);
}

#ifdef PROTO_OK
PTGNode PTGWebPage(PTGNode p1, PTGNode p2, PTGNode p3, PTGNode p4, PTGNode p5, PTGNode p6)
#else
PTGNode PTGWebPage(p1, p2, p3, p4, p5, p6)
PTGNode p1;
PTGNode p2;
PTGNode p3;
PTGNode p4;
PTGNode p5;
PTGNode p6;
#endif
{
	_PPTGWebPage n;

	n = (_PPTGWebPage)MALLOC(sizeof(struct _SPTGWebPage));
	n->_print = (_PTGProc)_PrPTGWebPage;
	n->p1 = p1;
	n->p2 = p2;
	n->p3 = p3;
	n->p4 = p4;
	n->p5 = p5;
	n->p6 = p6;
	return (PTGNode)n;
}

/* Implementation of Pattern WebPageProtector */

typedef struct _SPTGWebPageProtector{
	_PTGProc _print;
	PTGNode p1;
	PTGNode p2;
	PTGNode p3;
} * _PPTGWebPageProtector;

#ifdef PROTO_OK
static void _PrPTGWebPageProtector(_PPTGWebPageProtector n)
#else
static void _PrPTGWebPageProtector(n)
	_PPTGWebPageProtector n;
#endif
{
	PTG_OUTPUT_STRING(f, "$permission_list = '");
	n->p1->_print(n->p1);
	PTG_OUTPUT_STRING(f, "';");
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "if ((strlen($permission_list) > 0) && (!isset($_SESSION['user_nickname'])))");
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "{");
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "header('Location: ");
	n->p2->_print(n->p2);
	PTG_OUTPUT_STRING(f, "');");
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "} else {");
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "if ($_SESSION['Rollen_liste'] != $permission_list && $permission_list!='')");
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "{");
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "$ben_rollen = explode(' ', $_SESSION['Rollen_liste']);");
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "$web_rollen = explode(' ', $permission_list);");
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "$result = array_intersect($ben_rollen, $web_rollen);");
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "$result1 = count($result);");
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "if ($result1 != 1) {");
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "header('Location: ");
	n->p2->_print(n->p2);
	PTG_OUTPUT_STRING(f, "');");
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "}");
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "} else {");
	IndentNewLine(f);
	n->p3->_print(n->p3);
	PTG_OUTPUT_STRING(f, "}");
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "}");
	IndentNewLine(f);
}

#ifdef PROTO_OK
PTGNode PTGWebPageProtector(PTGNode p1, PTGNode p2, PTGNode p3)
#else
PTGNode PTGWebPageProtector(p1, p2, p3)
PTGNode p1;
PTGNode p2;
PTGNode p3;
#endif
{
	_PPTGWebPageProtector n;

	n = (_PPTGWebPageProtector)MALLOC(sizeof(struct _SPTGWebPageProtector));
	n->_print = (_PTGProc)_PrPTGWebPageProtector;
	n->p1 = p1;
	n->p2 = p2;
	n->p3 = p3;
	return (PTGNode)n;
}

/* Implementation of Pattern WebPageHeader */

typedef struct _SPTGWebPageHeader{
	_PTGProc _print;
} * _PPTGWebPageHeader;

#ifdef PROTO_OK
static void _PrPTGWebPageHeader(_PPTGWebPageHeader n)
#else
static void _PrPTGWebPageHeader(n)
	_PPTGWebPageHeader n;
#endif
{
	PTG_OUTPUT_STRING(f, "if ($_SERVER['SERVER_PROTOCOL'] == 'HTTP/1.1' &&");
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "isset($_SERVER['HTTP_ACCEPT']) &&");
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "preg_match('|application/xhtml\\+xml(?!\\s*;\\s*q=0)|', $_SERVER['HTTP_ACCEPT'])) {");
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "header('Content-Type: application/xhtml+xml; charset=utf-8');");
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "header('Vary: Negotiate,Accept');");
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "} else {");
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "header('Content-Type: text/html; charset=utf-8');");
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "header('Vary: Negotiate,Accept');");
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "}");
	IndentNewLine(f);
}

static struct _SPTGWebPageHeader _sptgWebPageHeader = { (_PTGProc) _PrPTGWebPageHeader };

#ifdef PROTO_OK
PTGNode PTGWebPageHeader(void)
#else
PTGNode PTGWebPageHeader()
#endif
{
	return (PTGNode)(&_sptgWebPageHeader);
}

/* Implementation of Pattern WebPageBody */

typedef struct _SPTGWebPageBody{
	_PTGProc _print;
	PTGNode p1;
	PTGNode p2;
} * _PPTGWebPageBody;

#ifdef PROTO_OK
static void _PrPTGWebPageBody(_PPTGWebPageBody n)
#else
static void _PrPTGWebPageBody(n)
	_PPTGWebPageBody n;
#endif
{
	PTG_OUTPUT_STRING(f, "<body class='");
	n->p2->_print(n->p2);
	PTG_OUTPUT_STRING(f, "'>");
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "<?php Include('navigationbar.php');?>");
	IndentNewLine(f);
	n->p1->_print(n->p1);
	PTG_OUTPUT_STRING(f, "<!-- Ausgeben des OutputBuffers -->");
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "<?php ob_flush(); ?>");
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "</body>");
	IndentNewLine(f);
}

#ifdef PROTO_OK
PTGNode PTGWebPageBody(PTGNode p1, PTGNode p2)
#else
PTGNode PTGWebPageBody(p1, p2)
PTGNode p1;
PTGNode p2;
#endif
{
	_PPTGWebPageBody n;

	n = (_PPTGWebPageBody)MALLOC(sizeof(struct _SPTGWebPageBody));
	n->_print = (_PTGProc)_PrPTGWebPageBody;
	n->p1 = p1;
	n->p2 = p2;
	return (PTGNode)n;
}

/* Implementation of Pattern WebPageBodyProtected */

typedef struct _SPTGWebPageBodyProtected{
	_PTGProc _print;
	PTGNode p1;
	PTGNode p2;
} * _PPTGWebPageBodyProtected;

#ifdef PROTO_OK
static void _PrPTGWebPageBodyProtected(_PPTGWebPageBodyProtected n)
#else
static void _PrPTGWebPageBodyProtected(n)
	_PPTGWebPageBodyProtected n;
#endif
{
	PTG_OUTPUT_STRING(f, "<?php ");
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "if (isset($_SESSION['isuser']))");
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "{?>");
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "<body class='");
	n->p2->_print(n->p2);
	PTG_OUTPUT_STRING(f, "'>");
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "<?php Include('navigationbar.php');?>");
	IndentNewLine(f);
	n->p1->_print(n->p1);
	PTG_OUTPUT_STRING(f, "<!-- Ausgeben des OutputBuffers -->");
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "<?php ob_flush(); ?>");
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "</body>");
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "<?php } else {");
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "echo \"<meta http-equiv='refresh' content='0; url=errorlinkpage.php'>\";");
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "echo \"</meta>\"; ");
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "}");
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "?>");
	IndentNewLine(f);
}

#ifdef PROTO_OK
PTGNode PTGWebPageBodyProtected(PTGNode p1, PTGNode p2)
#else
PTGNode PTGWebPageBodyProtected(p1, p2)
PTGNode p1;
PTGNode p2;
#endif
{
	_PPTGWebPageBodyProtected n;

	n = (_PPTGWebPageBodyProtected)MALLOC(sizeof(struct _SPTGWebPageBodyProtected));
	n->_print = (_PTGProc)_PrPTGWebPageBodyProtected;
	n->p1 = p1;
	n->p2 = p2;
	return (PTGNode)n;
}

/* Implementation of Pattern Frame */

typedef struct _SPTGFrame{
	_PTGProc _print;
	PTGNode p1;
	PTGNode p2;
} * _PPTGFrame;

#ifdef PROTO_OK
static void _PrPTGFrame(_PPTGFrame n)
#else
static void _PrPTGFrame(n)
	_PPTGFrame n;
#endif
{
	PTG_OUTPUT_STRING(f, "<div class='");
	n->p1->_print(n->p1);
	PTG_OUTPUT_STRING(f, "' >");
	IndentNewLine(f);
	n->p2->_print(n->p2);
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "</div>");
	IndentNewLine(f);
}

#ifdef PROTO_OK
PTGNode PTGFrame(PTGNode p1, PTGNode p2)
#else
PTGNode PTGFrame(p1, p2)
PTGNode p1;
PTGNode p2;
#endif
{
	_PPTGFrame n;

	n = (_PPTGFrame)MALLOC(sizeof(struct _SPTGFrame));
	n->_print = (_PTGProc)_PrPTGFrame;
	n->p1 = p1;
	n->p2 = p2;
	return (PTGNode)n;
}

/* Implementation of Pattern FrameRef */

typedef struct _SPTGFrameRef{
	_PTGProc _print;
	PTGNode p1;
} * _PPTGFrameRef;

#ifdef PROTO_OK
static void _PrPTGFrameRef(_PPTGFrameRef n)
#else
static void _PrPTGFrameRef(n)
	_PPTGFrameRef n;
#endif
{
	PTG_OUTPUT_STRING(f, "<?php Include('");
	n->p1->_print(n->p1);
	PTG_OUTPUT_STRING(f, "');?>");
	IndentNewLine(f);
}

#ifdef PROTO_OK
PTGNode PTGFrameRef(PTGNode p1)
#else
PTGNode PTGFrameRef(p1)
PTGNode p1;
#endif
{
	_PPTGFrameRef n;

	n = (_PPTGFrameRef)MALLOC(sizeof(struct _SPTGFrameRef));
	n->_print = (_PTGProc)_PrPTGFrameRef;
	n->p1 = p1;
	return (PTGNode)n;
}

/* Implementation of Pattern WebAction */

typedef struct _SPTGWebAction{
	_PTGProc _print;
	PTGNode p1;
	PTGNode p2;
	PTGNode p3;
} * _PPTGWebAction;

#ifdef PROTO_OK
static void _PrPTGWebAction(_PPTGWebAction n)
#else
static void _PrPTGWebAction(n)
	_PPTGWebAction n;
#endif
{
	PTG_OUTPUT_STRING(f, "<?php");
	IndentIncr(f);
	IndentNewLine(f);
	n->p1->_print(n->p1);
	IndentNewLine(f);
	n->p2->_print(n->p2);
	IndentNewLine(f);
	n->p3->_print(n->p3);
	IndentDecr(f);
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "?>");
	IndentNewLine(f);
}

#ifdef PROTO_OK
PTGNode PTGWebAction(PTGNode p1, PTGNode p2, PTGNode p3)
#else
PTGNode PTGWebAction(p1, p2, p3)
PTGNode p1;
PTGNode p2;
PTGNode p3;
#endif
{
	_PPTGWebAction n;

	n = (_PPTGWebAction)MALLOC(sizeof(struct _SPTGWebAction));
	n->_print = (_PTGProc)_PrPTGWebAction;
	n->p1 = p1;
	n->p2 = p2;
	n->p3 = p3;
	return (PTGNode)n;
}

/* Implementation of Pattern PHPOperation */

typedef struct _SPTGPHPOperation{
	_PTGProc _print;
	PTGNode p1;
	PTGNode p2;
} * _PPTGPHPOperation;

#ifdef PROTO_OK
static void _PrPTGPHPOperation(_PPTGPHPOperation n)
#else
static void _PrPTGPHPOperation(n)
	_PPTGPHPOperation n;
#endif
{
	n->p2->_print(n->p2);
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "/* Start des benutzerdefinierten PHP-Codes */");
	IndentNewLine(f);
	IndentNewLine(f);
	n->p1->_print(n->p1);
	IndentNewLine(f);
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "/* Ende des benutzerdefinierten PHP-Codes */");
	IndentNewLine(f);
}

#ifdef PROTO_OK
PTGNode PTGPHPOperation(PTGNode p1, PTGNode p2)
#else
PTGNode PTGPHPOperation(p1, p2)
PTGNode p1;
PTGNode p2;
#endif
{
	_PPTGPHPOperation n;

	n = (_PPTGPHPOperation)MALLOC(sizeof(struct _SPTGPHPOperation));
	n->_print = (_PTGProc)_PrPTGPHPOperation;
	n->p1 = p1;
	n->p2 = p2;
	return (PTGNode)n;
}

/* Implementation of Pattern DataBaseOperation */

typedef struct _SPTGDataBaseOperation{
	_PTGProc _print;
	PTGNode p1;
	PTGNode p2;
	PTGNode p3;
} * _PPTGDataBaseOperation;

#ifdef PROTO_OK
static void _PrPTGDataBaseOperation(_PPTGDataBaseOperation n)
#else
static void _PrPTGDataBaseOperation(n)
	_PPTGDataBaseOperation n;
#endif
{
	PTG_OUTPUT_STRING(f, "/* Definition der DataBaseOperation */");
	IndentNewLine(f);
	n->p1->_print(n->p1);
	IndentNewLine(f);
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "/* Aufruf der DataBaseOperation */");
	IndentNewLine(f);
	n->p2->_print(n->p2);
	PTG_OUTPUT_STRING(f, "(");
	n->p3->_print(n->p3);
	PTG_OUTPUT_STRING(f, ");");
	IndentNewLine(f);
}

#ifdef PROTO_OK
PTGNode PTGDataBaseOperation(PTGNode p1, PTGNode p2, PTGNode p3)
#else
PTGNode PTGDataBaseOperation(p1, p2, p3)
PTGNode p1;
PTGNode p2;
PTGNode p3;
#endif
{
	_PPTGDataBaseOperation n;

	n = (_PPTGDataBaseOperation)MALLOC(sizeof(struct _SPTGDataBaseOperation));
	n->_print = (_PTGProc)_PrPTGDataBaseOperation;
	n->p1 = p1;
	n->p2 = p2;
	n->p3 = p3;
	return (PTGNode)n;
}

/* Implementation of Pattern Redirect */

typedef struct _SPTGRedirect{
	_PTGProc _print;
	PTGNode p1;
} * _PPTGRedirect;

#ifdef PROTO_OK
static void _PrPTGRedirect(_PPTGRedirect n)
#else
static void _PrPTGRedirect(n)
	_PPTGRedirect n;
#endif
{
	PTG_OUTPUT_STRING(f, "$params = preg_replace(\'/\\/.*\?\\?/\', '?', $_SERVER[\'REQUEST_URI\']);");
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "$params = preg_replace(\'/(\\&|\\?)Submit=Submit/\', \'\', $params);");
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "header(\'HTTP/1.1 303 See Other\');");
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "header(\"Location: http://\" . $_SERVER['HTTP_HOST'] . dirname($_SERVER['PHP_SELF']) . \"/");
	n->p1->_print(n->p1);
	PTG_OUTPUT_STRING(f, "\" . $params );");
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "ob_flush();");
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "exit;");
}

#ifdef PROTO_OK
PTGNode PTGRedirect(PTGNode p1)
#else
PTGNode PTGRedirect(p1)
PTGNode p1;
#endif
{
	_PPTGRedirect n;

	n = (_PPTGRedirect)MALLOC(sizeof(struct _SPTGRedirect));
	n->_print = (_PTGProc)_PrPTGRedirect;
	n->p1 = p1;
	return (PTGNode)n;
}

/* Implementation of Pattern Include */

typedef struct _SPTGInclude{
	_PTGProc _print;
	PTGNode p1;
} * _PPTGInclude;

#ifdef PROTO_OK
static void _PrPTGInclude(_PPTGInclude n)
#else
static void _PrPTGInclude(n)
	_PPTGInclude n;
#endif
{
	PTG_OUTPUT_STRING(f, "include(\"");
	n->p1->_print(n->p1);
	PTG_OUTPUT_STRING(f, "\");");
	IndentNewLine(f);
}

#ifdef PROTO_OK
PTGNode PTGInclude(PTGNode p1)
#else
PTGNode PTGInclude(p1)
PTGNode p1;
#endif
{
	_PPTGInclude n;

	n = (_PPTGInclude)MALLOC(sizeof(struct _SPTGInclude));
	n->_print = (_PTGProc)_PrPTGInclude;
	n->p1 = p1;
	return (PTGNode)n;
}

/* Implementation of Pattern ActionHandler */

typedef struct _SPTGActionHandler{
	_PTGProc _print;
	PTGNode p1;
} * _PPTGActionHandler;

#ifdef PROTO_OK
static void _PrPTGActionHandler(_PPTGActionHandler n)
#else
static void _PrPTGActionHandler(n)
	_PPTGActionHandler n;
#endif
{
	PTG_OUTPUT_STRING(f, "/* Redirect zur TargetPage */");
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "header(\'HTTP/1.1 303 See Other\');");
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "header(\"Location: http://\" . $_SERVER['HTTP_HOST'] . dirname($_SERVER['PHP_SELF']) . \"/");
	n->p1->_print(n->p1);
	PTG_OUTPUT_STRING(f, ".php\");");
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "ob_flush();");
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "exit;");
	IndentNewLine(f);
}

#ifdef PROTO_OK
PTGNode PTGActionHandler(PTGNode p1)
#else
PTGNode PTGActionHandler(p1)
PTGNode p1;
#endif
{
	_PPTGActionHandler n;

	n = (_PPTGActionHandler)MALLOC(sizeof(struct _SPTGActionHandler));
	n->_print = (_PTGProc)_PrPTGActionHandler;
	n->p1 = p1;
	return (PTGNode)n;
}

/* Implementation of Pattern ActionHandler2 */

typedef struct _SPTGActionHandler2{
	_PTGProc _print;
	PTGNode p1;
} * _PPTGActionHandler2;

#ifdef PROTO_OK
static void _PrPTGActionHandler2(_PPTGActionHandler2 n)
#else
static void _PrPTGActionHandler2(n)
	_PPTGActionHandler2 n;
#endif
{
	PTG_OUTPUT_STRING(f, "/* ErrorHandler-Funktion fr die folgende PHPOperation */");
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "function ");
	n->p1->_print(n->p1);
	PTG_OUTPUT_STRING(f, "ErrorHandler($errno, $errstr, $errfile, $errline)");
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "{");
	IndentIncr(f);
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "switch ($errno) {");
	IndentIncr(f);
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "case E_USER_ERROR:");
	IndentIncr(f);
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "header(\'HTTP/1.1 303 See Other\');");
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "header(\"Location: http://\" . $_SERVER['HTTP_HOST'] . dirname($_SERVER['PHP_SELF']) . \"/");
	n->p1->_print(n->p1);
	PTG_OUTPUT_STRING(f, ".php?errno=$errno&errstr=$errstr&errfile=$errfile&errline=$errline\");");
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "ob_flush();");
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "exit;");
	IndentDecr(f);
	IndentDecr(f);
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "}");
	IndentDecr(f);
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "}");
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "/* Aktivieren des ErrorHandlers */");
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "set_error_handler(\"");
	n->p1->_print(n->p1);
	PTG_OUTPUT_STRING(f, "ErrorHandler\");");
	IndentNewLine(f);
	IndentNewLine(f);
}

#ifdef PROTO_OK
PTGNode PTGActionHandler2(PTGNode p1)
#else
PTGNode PTGActionHandler2(p1)
PTGNode p1;
#endif
{
	_PPTGActionHandler2 n;

	n = (_PPTGActionHandler2)MALLOC(sizeof(struct _SPTGActionHandler2));
	n->_print = (_PTGProc)_PrPTGActionHandler2;
	n->p1 = p1;
	return (PTGNode)n;
}

/* Implementation of Pattern Formular */

typedef struct _SPTGFormular{
	_PTGProc _print;
	PTGNode p1;
	PTGNode p2;
	PTGNode p3;
	PTGNode p4;
	PTGNode p5;
	PTGNode p6;
	PTGNode p7;
} * _PPTGFormular;

#ifdef PROTO_OK
static void _PrPTGFormular(_PPTGFormular n)
#else
static void _PrPTGFormular(n)
	_PPTGFormular n;
#endif
{
	PTG_OUTPUT_STRING(f, "<!-- Start Formular -->");
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "<?php");
	IndentIncr(f);
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "/* Konsistenz-Checks */");
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "if (!empty($_REQUEST['Submit'])) {");
	IndentIncr(f);
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "$condresult = array();");
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "/* Definition von Konsistenzcheck-Funktionen */");
	IndentNewLine(f);
	n->p7->_print(n->p7);
	IndentNewLine(f);
	n->p5->_print(n->p5);
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "if (count($condresult) > 0) {");
	IndentIncr(f);
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "$keys = array_keys($condresult);");
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "echo \"<table cellpadding='3'>\n\";");
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "for ($i = 0; $i < count($keys); $i++) {");
	IndentIncr(f);
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "$key = $keys[$i];");
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "echo \"<tr><td>Fehler in Feld \'$key\': $condresult[$key]</td></tr>\n\";");
	IndentDecr(f);
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "}");
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "echo \"</table><br clear='all'></br>\n\";");
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "} else {");
	IndentIncr(f);
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "/* Einbinden der aufgerufenen Action */");
	IndentNewLine(f);
	n->p2->_print(n->p2);
	IndentDecr(f);
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "}");
	IndentDecr(f);
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "}");
	IndentDecr(f);
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "?>");
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "<form class='");
	n->p1->_print(n->p1);
	PTG_OUTPUT_STRING(f, "' action='");
	n->p6->_print(n->p6);
	PTG_OUTPUT_STRING(f, "' method='");
	n->p3->_print(n->p3);
	PTG_OUTPUT_STRING(f, "'>");
	IndentIncr(f);
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "<table cellpadding='3'>");
	IndentNewLine(f);
	n->p4->_print(n->p4);
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "<tr><td></td><td><input type='submit' name='Submit' value='Submit'></input>");
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "<input type='reset' name='Reset' value='Reset'></input></td></tr>");
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "</table><br clear='all'></br>");
	IndentDecr(f);
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "</form>");
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "<!-- Ende Formular -->");
	IndentNewLine(f);
}

#ifdef PROTO_OK
PTGNode PTGFormular(PTGNode p1, PTGNode p2, PTGNode p3, PTGNode p4, PTGNode p5, PTGNode p6, PTGNode p7)
#else
PTGNode PTGFormular(p1, p2, p3, p4, p5, p6, p7)
PTGNode p1;
PTGNode p2;
PTGNode p3;
PTGNode p4;
PTGNode p5;
PTGNode p6;
PTGNode p7;
#endif
{
	_PPTGFormular n;

	n = (_PPTGFormular)MALLOC(sizeof(struct _SPTGFormular));
	n->_print = (_PTGProc)_PrPTGFormular;
	n->p1 = p1;
	n->p2 = p2;
	n->p3 = p3;
	n->p4 = p4;
	n->p5 = p5;
	n->p6 = p6;
	n->p7 = p7;
	return (PTGNode)n;
}

/* Implementation of Pattern FormularCondition */

typedef struct _SPTGFormularCondition{
	_PTGProc _print;
	PTGNode p1;
	PTGNode p2;
} * _PPTGFormularCondition;

#ifdef PROTO_OK
static void _PrPTGFormularCondition(_PPTGFormularCondition n)
#else
static void _PrPTGFormularCondition(n)
	_PPTGFormularCondition n;
#endif
{
	PTG_OUTPUT_STRING(f, "/* Aufruf des Konsistenzchecks */");
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "$condresult = $condresult + ");
	n->p1->_print(n->p1);
	PTG_OUTPUT_STRING(f, "(");
	n->p2->_print(n->p2);
	PTG_OUTPUT_STRING(f, ");");
	IndentNewLine(f);
}

#ifdef PROTO_OK
PTGNode PTGFormularCondition(PTGNode p1, PTGNode p2)
#else
PTGNode PTGFormularCondition(p1, p2)
PTGNode p1;
PTGNode p2;
#endif
{
	_PPTGFormularCondition n;

	n = (_PPTGFormularCondition)MALLOC(sizeof(struct _SPTGFormularCondition));
	n->_print = (_PTGProc)_PrPTGFormularCondition;
	n->p1 = p1;
	n->p2 = p2;
	return (PTGNode)n;
}

/* Implementation of Pattern FormularConditionDefinition */

typedef struct _SPTGFormularConditionDefinition{
	_PTGProc _print;
	PTGNode p1;
	PTGNode p2;
	PTGNode p3;
} * _PPTGFormularConditionDefinition;

#ifdef PROTO_OK
static void _PrPTGFormularConditionDefinition(_PPTGFormularConditionDefinition n)
#else
static void _PrPTGFormularConditionDefinition(n)
	_PPTGFormularConditionDefinition n;
#endif
{
	PTG_OUTPUT_STRING(f, "function ");
	n->p1->_print(n->p1);
	PTG_OUTPUT_STRING(f, "(");
	n->p2->_print(n->p2);
	PTG_OUTPUT_STRING(f, ") {");
	IndentIncr(f);
	IndentNewLine(f);
	n->p3->_print(n->p3);
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "if (!empty($error)) { return array($fieldName => $error); }");
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "else { return array(); }");
	IndentDecr(f);
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "}");
	IndentNewLine(f);
}

#ifdef PROTO_OK
PTGNode PTGFormularConditionDefinition(PTGNode p1, PTGNode p2, PTGNode p3)
#else
PTGNode PTGFormularConditionDefinition(p1, p2, p3)
PTGNode p1;
PTGNode p2;
PTGNode p3;
#endif
{
	_PPTGFormularConditionDefinition n;

	n = (_PPTGFormularConditionDefinition)MALLOC(sizeof(struct _SPTGFormularConditionDefinition));
	n->_print = (_PTGProc)_PrPTGFormularConditionDefinition;
	n->p1 = p1;
	n->p2 = p2;
	n->p3 = p3;
	return (PTGNode)n;
}

/* Implementation of Pattern FieldName */

typedef struct _SPTGFieldName{
	_PTGProc _print;
	PTGNode p1;
} * _PPTGFieldName;

#ifdef PROTO_OK
static void _PrPTGFieldName(_PPTGFieldName n)
#else
static void _PrPTGFieldName(n)
	_PPTGFieldName n;
#endif
{
	PTG_OUTPUT_STRING(f, "\"");
	n->p1->_print(n->p1);
	PTG_OUTPUT_STRING(f, "\"");
}

#ifdef PROTO_OK
PTGNode PTGFieldName(PTGNode p1)
#else
PTGNode PTGFieldName(p1)
PTGNode p1;
#endif
{
	_PPTGFieldName n;

	n = (_PPTGFieldName)MALLOC(sizeof(struct _SPTGFieldName));
	n->_print = (_PTGProc)_PrPTGFieldName;
	n->p1 = p1;
	return (PTGNode)n;
}

/* Implementation of Pattern FieldValue */

typedef struct _SPTGFieldValue{
	_PTGProc _print;
	PTGNode p1;
} * _PPTGFieldValue;

#ifdef PROTO_OK
static void _PrPTGFieldValue(_PPTGFieldValue n)
#else
static void _PrPTGFieldValue(n)
	_PPTGFieldValue n;
#endif
{
	PTG_OUTPUT_STRING(f, "$_REQUEST['");
	n->p1->_print(n->p1);
	PTG_OUTPUT_STRING(f, "']");
}

#ifdef PROTO_OK
PTGNode PTGFieldValue(PTGNode p1)
#else
PTGNode PTGFieldValue(p1)
PTGNode p1;
#endif
{
	_PPTGFieldValue n;

	n = (_PPTGFieldValue)MALLOC(sizeof(struct _SPTGFieldValue));
	n->_print = (_PTGProc)_PrPTGFieldValue;
	n->p1 = p1;
	return (PTGNode)n;
}

/* Implementation of Pattern FormularCheckboxGroupElement */

typedef struct _SPTGFormularCheckboxGroupElement{
	_PTGProc _print;
	PTGNode p1;
} * _PPTGFormularCheckboxGroupElement;

#ifdef PROTO_OK
static void _PrPTGFormularCheckboxGroupElement(_PPTGFormularCheckboxGroupElement n)
#else
static void _PrPTGFormularCheckboxGroupElement(n)
	_PPTGFormularCheckboxGroupElement n;
#endif
{
	n->p1->_print(n->p1);
	IndentNewLine(f);
}

#ifdef PROTO_OK
PTGNode PTGFormularCheckboxGroupElement(PTGNode p1)
#else
PTGNode PTGFormularCheckboxGroupElement(p1)
PTGNode p1;
#endif
{
	_PPTGFormularCheckboxGroupElement n;

	n = (_PPTGFormularCheckboxGroupElement)MALLOC(sizeof(struct _SPTGFormularCheckboxGroupElement));
	n->_print = (_PTGProc)_PrPTGFormularCheckboxGroupElement;
	n->p1 = p1;
	return (PTGNode)n;
}

/* No Implementation of Pattern FormularRadioGroupElement needed */
/* See FormularCheckboxGroupElement for implementation */

/* Implementation of Pattern FormularTextElement */

typedef struct _SPTGFormularTextElement{
	_PTGProc _print;
	PTGNode p1;
	PTGNode p2;
	PTGNode p3;
	PTGNode p4;
} * _PPTGFormularTextElement;

#ifdef PROTO_OK
static void _PrPTGFormularTextElement(_PPTGFormularTextElement n)
#else
static void _PrPTGFormularTextElement(n)
	_PPTGFormularTextElement n;
#endif
{
	PTG_OUTPUT_STRING(f, "<tr><td><label for='");
	n->p2->_print(n->p2);
	PTG_OUTPUT_STRING(f, "'>");
	n->p1->_print(n->p1);
	PTG_OUTPUT_STRING(f, "</label></td>");
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "<td><input id='");
	n->p2->_print(n->p2);
	PTG_OUTPUT_STRING(f, "' type='text' name='");
	n->p2->_print(n->p2);
	PTG_OUTPUT_STRING(f, "' value='");
	if (n->p2 != PTGNULL && n->p1 != PTGNULL && n->p2 != PTGNULL && n->p2 != PTGNULL && n->p3 != PTGNULL && n->p4 != PTGNULL)
	{
		PTG_OUTPUT_STRING(f, "<?php echo $i");
	}
	n->p3->_print(n->p3);
	if (n->p2 != PTGNULL && n->p1 != PTGNULL && n->p2 != PTGNULL && n->p2 != PTGNULL && n->p3 != PTGNULL && n->p4 != PTGNULL)
	{
		PTG_OUTPUT_STRING(f, "; ?>");
	}
	PTG_OUTPUT_STRING(f, "'></input></td>");
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "<td>");
	n->p4->_print(n->p4);
	PTG_OUTPUT_STRING(f, "</td></tr>");
	IndentNewLine(f);
}

#ifdef PROTO_OK
PTGNode PTGFormularTextElement(PTGNode p1, PTGNode p2, PTGNode p3, PTGNode p4)
#else
PTGNode PTGFormularTextElement(p1, p2, p3, p4)
PTGNode p1;
PTGNode p2;
PTGNode p3;
PTGNode p4;
#endif
{
	_PPTGFormularTextElement n;

	n = (_PPTGFormularTextElement)MALLOC(sizeof(struct _SPTGFormularTextElement));
	n->_print = (_PTGProc)_PrPTGFormularTextElement;
	n->p1 = p1;
	n->p2 = p2;
	n->p3 = p3;
	n->p4 = p4;
	return (PTGNode)n;
}

/* Implementation of Pattern FormularTextAreaElement */

typedef struct _SPTGFormularTextAreaElement{
	_PTGProc _print;
	PTGNode p1;
	PTGNode p2;
	PTGNode p3;
	PTGNode p4;
} * _PPTGFormularTextAreaElement;

#ifdef PROTO_OK
static void _PrPTGFormularTextAreaElement(_PPTGFormularTextAreaElement n)
#else
static void _PrPTGFormularTextAreaElement(n)
	_PPTGFormularTextAreaElement n;
#endif
{
	PTG_OUTPUT_STRING(f, "<tr><td><label for='");
	n->p2->_print(n->p2);
	PTG_OUTPUT_STRING(f, "'>");
	n->p1->_print(n->p1);
	PTG_OUTPUT_STRING(f, "</label></td>");
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "<td><textarea id='");
	n->p2->_print(n->p2);
	PTG_OUTPUT_STRING(f, "' name='");
	n->p2->_print(n->p2);
	PTG_OUTPUT_STRING(f, "'>");
	if (n->p2 != PTGNULL && n->p1 != PTGNULL && n->p2 != PTGNULL && n->p2 != PTGNULL && n->p3 != PTGNULL && n->p4 != PTGNULL)
	{
		PTG_OUTPUT_STRING(f, "<?php echo $");
	}
	n->p3->_print(n->p3);
	if (n->p2 != PTGNULL && n->p1 != PTGNULL && n->p2 != PTGNULL && n->p2 != PTGNULL && n->p3 != PTGNULL && n->p4 != PTGNULL)
	{
		PTG_OUTPUT_STRING(f, "; ?>");
	}
	PTG_OUTPUT_STRING(f, "</textarea></td>");
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "<td>");
	n->p4->_print(n->p4);
	PTG_OUTPUT_STRING(f, "</td></tr>");
	IndentNewLine(f);
}

#ifdef PROTO_OK
PTGNode PTGFormularTextAreaElement(PTGNode p1, PTGNode p2, PTGNode p3, PTGNode p4)
#else
PTGNode PTGFormularTextAreaElement(p1, p2, p3, p4)
PTGNode p1;
PTGNode p2;
PTGNode p3;
PTGNode p4;
#endif
{
	_PPTGFormularTextAreaElement n;

	n = (_PPTGFormularTextAreaElement)MALLOC(sizeof(struct _SPTGFormularTextAreaElement));
	n->_print = (_PTGProc)_PrPTGFormularTextAreaElement;
	n->p1 = p1;
	n->p2 = p2;
	n->p3 = p3;
	n->p4 = p4;
	return (PTGNode)n;
}

/* Implementation of Pattern FormularPasswordElement */

typedef struct _SPTGFormularPasswordElement{
	_PTGProc _print;
	PTGNode p1;
	PTGNode p2;
	PTGNode p3;
	PTGNode p4;
} * _PPTGFormularPasswordElement;

#ifdef PROTO_OK
static void _PrPTGFormularPasswordElement(_PPTGFormularPasswordElement n)
#else
static void _PrPTGFormularPasswordElement(n)
	_PPTGFormularPasswordElement n;
#endif
{
	PTG_OUTPUT_STRING(f, "<tr><td><label for='");
	n->p2->_print(n->p2);
	PTG_OUTPUT_STRING(f, "'>");
	n->p1->_print(n->p1);
	PTG_OUTPUT_STRING(f, "</label></td>");
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "<td><input id='");
	n->p2->_print(n->p2);
	PTG_OUTPUT_STRING(f, "' type='password' name='");
	n->p2->_print(n->p2);
	PTG_OUTPUT_STRING(f, "' value='");
	if (n->p2 != PTGNULL && n->p1 != PTGNULL && n->p2 != PTGNULL && n->p2 != PTGNULL && n->p3 != PTGNULL && n->p4 != PTGNULL)
	{
		PTG_OUTPUT_STRING(f, "<?php echo $");
	}
	n->p3->_print(n->p3);
	if (n->p2 != PTGNULL && n->p1 != PTGNULL && n->p2 != PTGNULL && n->p2 != PTGNULL && n->p3 != PTGNULL && n->p4 != PTGNULL)
	{
		PTG_OUTPUT_STRING(f, " ?>");
	}
	PTG_OUTPUT_STRING(f, "'></input></td>");
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "<td>");
	n->p4->_print(n->p4);
	PTG_OUTPUT_STRING(f, "</td></tr>");
	IndentNewLine(f);
}

#ifdef PROTO_OK
PTGNode PTGFormularPasswordElement(PTGNode p1, PTGNode p2, PTGNode p3, PTGNode p4)
#else
PTGNode PTGFormularPasswordElement(p1, p2, p3, p4)
PTGNode p1;
PTGNode p2;
PTGNode p3;
PTGNode p4;
#endif
{
	_PPTGFormularPasswordElement n;

	n = (_PPTGFormularPasswordElement)MALLOC(sizeof(struct _SPTGFormularPasswordElement));
	n->_print = (_PTGProc)_PrPTGFormularPasswordElement;
	n->p1 = p1;
	n->p2 = p2;
	n->p3 = p3;
	n->p4 = p4;
	return (PTGNode)n;
}

/* Implementation of Pattern FormularCheckboxElement */

typedef struct _SPTGFormularCheckboxElement{
	_PTGProc _print;
	PTGNode p1;
	PTGNode p2;
	PTGNode p3;
} * _PPTGFormularCheckboxElement;

#ifdef PROTO_OK
static void _PrPTGFormularCheckboxElement(_PPTGFormularCheckboxElement n)
#else
static void _PrPTGFormularCheckboxElement(n)
	_PPTGFormularCheckboxElement n;
#endif
{
	PTG_OUTPUT_STRING(f, "<tr><td><label for='");
	n->p1->_print(n->p1);
	PTG_OUTPUT_STRING(f, "'>");
	n->p3->_print(n->p3);
	PTG_OUTPUT_STRING(f, "</label></td>");
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "<td><input id='");
	n->p1->_print(n->p1);
	PTG_OUTPUT_STRING(f, "' type='checkbox' name='");
	n->p1->_print(n->p1);
	PTG_OUTPUT_STRING(f, "' value='");
	n->p2->_print(n->p2);
	PTG_OUTPUT_STRING(f, "'> </input></td></tr>");
	IndentNewLine(f);
}

#ifdef PROTO_OK
PTGNode PTGFormularCheckboxElement(PTGNode p1, PTGNode p2, PTGNode p3)
#else
PTGNode PTGFormularCheckboxElement(p1, p2, p3)
PTGNode p1;
PTGNode p2;
PTGNode p3;
#endif
{
	_PPTGFormularCheckboxElement n;

	n = (_PPTGFormularCheckboxElement)MALLOC(sizeof(struct _SPTGFormularCheckboxElement));
	n->_print = (_PTGProc)_PrPTGFormularCheckboxElement;
	n->p1 = p1;
	n->p2 = p2;
	n->p3 = p3;
	return (PTGNode)n;
}

/* Implementation of Pattern FormularRadioElement */

typedef struct _SPTGFormularRadioElement{
	_PTGProc _print;
	PTGNode p1;
	PTGNode p2;
	PTGNode p3;
} * _PPTGFormularRadioElement;

#ifdef PROTO_OK
static void _PrPTGFormularRadioElement(_PPTGFormularRadioElement n)
#else
static void _PrPTGFormularRadioElement(n)
	_PPTGFormularRadioElement n;
#endif
{
	PTG_OUTPUT_STRING(f, "<tr><td><label for='");
	n->p1->_print(n->p1);
	PTG_OUTPUT_STRING(f, "'>");
	n->p3->_print(n->p3);
	PTG_OUTPUT_STRING(f, "</label></td>");
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "<td><input id='");
	n->p1->_print(n->p1);
	PTG_OUTPUT_STRING(f, "' type='radio' name='");
	n->p1->_print(n->p1);
	PTG_OUTPUT_STRING(f, "' value='");
	n->p2->_print(n->p2);
	PTG_OUTPUT_STRING(f, "'> </input></td></tr>");
	IndentNewLine(f);
}

#ifdef PROTO_OK
PTGNode PTGFormularRadioElement(PTGNode p1, PTGNode p2, PTGNode p3)
#else
PTGNode PTGFormularRadioElement(p1, p2, p3)
PTGNode p1;
PTGNode p2;
PTGNode p3;
#endif
{
	_PPTGFormularRadioElement n;

	n = (_PPTGFormularRadioElement)MALLOC(sizeof(struct _SPTGFormularRadioElement));
	n->_print = (_PTGProc)_PrPTGFormularRadioElement;
	n->p1 = p1;
	n->p2 = p2;
	n->p3 = p3;
	return (PTGNode)n;
}

/* Implementation of Pattern FormularSelectElement */

typedef struct _SPTGFormularSelectElement{
	_PTGProc _print;
	PTGNode p1;
	PTGNode p2;
	PTGNode p3;
	PTGNode p4;
	PTGNode p5;
	PTGNode p6;
} * _PPTGFormularSelectElement;

#ifdef PROTO_OK
static void _PrPTGFormularSelectElement(_PPTGFormularSelectElement n)
#else
static void _PrPTGFormularSelectElement(n)
	_PPTGFormularSelectElement n;
#endif
{
	PTG_OUTPUT_STRING(f, "<tr><td><p>");
	n->p1->_print(n->p1);
	PTG_OUTPUT_STRING(f, "</p></td>");
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "<td><select name='");
	n->p2->_print(n->p2);
	PTG_OUTPUT_STRING(f, "' size='");
	n->p3->_print(n->p3);
	PTG_OUTPUT_STRING(f, "' ");
	n->p4->_print(n->p4);
	PTG_OUTPUT_STRING(f, ">");
	IndentIncr(f);
	IndentNewLine(f);
	n->p5->_print(n->p5);
	IndentDecr(f);
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "</select></td>");
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "<td>");
	n->p6->_print(n->p6);
	PTG_OUTPUT_STRING(f, "</td></tr>");
	IndentNewLine(f);
}

#ifdef PROTO_OK
PTGNode PTGFormularSelectElement(PTGNode p1, PTGNode p2, PTGNode p3, PTGNode p4, PTGNode p5, PTGNode p6)
#else
PTGNode PTGFormularSelectElement(p1, p2, p3, p4, p5, p6)
PTGNode p1;
PTGNode p2;
PTGNode p3;
PTGNode p4;
PTGNode p5;
PTGNode p6;
#endif
{
	_PPTGFormularSelectElement n;

	n = (_PPTGFormularSelectElement)MALLOC(sizeof(struct _SPTGFormularSelectElement));
	n->_print = (_PTGProc)_PrPTGFormularSelectElement;
	n->p1 = p1;
	n->p2 = p2;
	n->p3 = p3;
	n->p4 = p4;
	n->p5 = p5;
	n->p6 = p6;
	return (PTGNode)n;
}

/* Implementation of Pattern FormularPHPScriptSelectOption */

typedef struct _SPTGFormularPHPScriptSelectOption{
	_PTGProc _print;
	PTGNode p1;
} * _PPTGFormularPHPScriptSelectOption;

#ifdef PROTO_OK
static void _PrPTGFormularPHPScriptSelectOption(_PPTGFormularPHPScriptSelectOption n)
#else
static void _PrPTGFormularPHPScriptSelectOption(n)
	_PPTGFormularPHPScriptSelectOption n;
#endif
{
	PTG_OUTPUT_STRING(f, "<?php");
	IndentIncr(f);
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "$values = array();");
	IndentNewLine(f);
	n->p1->_print(n->p1);
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "for ($i = 0; $i < count($values); $i++) {");
	IndentIncr(f);
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "echo \"<option> $values[$i] </option>\";");
	IndentDecr(f);
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "}");
	IndentDecr(f);
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "?>");
	IndentNewLine(f);
}

#ifdef PROTO_OK
PTGNode PTGFormularPHPScriptSelectOption(PTGNode p1)
#else
PTGNode PTGFormularPHPScriptSelectOption(p1)
PTGNode p1;
#endif
{
	_PPTGFormularPHPScriptSelectOption n;

	n = (_PPTGFormularPHPScriptSelectOption)MALLOC(sizeof(struct _SPTGFormularPHPScriptSelectOption));
	n->_print = (_PTGProc)_PrPTGFormularPHPScriptSelectOption;
	n->p1 = p1;
	return (PTGNode)n;
}

/* Implementation of Pattern FormularConstantSelectOption */

typedef struct _SPTGFormularConstantSelectOption{
	_PTGProc _print;
	PTGNode p1;
} * _PPTGFormularConstantSelectOption;

#ifdef PROTO_OK
static void _PrPTGFormularConstantSelectOption(_PPTGFormularConstantSelectOption n)
#else
static void _PrPTGFormularConstantSelectOption(n)
	_PPTGFormularConstantSelectOption n;
#endif
{
	PTG_OUTPUT_STRING(f, "<?php");
	IndentIncr(f);
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "$values = array(");
	n->p1->_print(n->p1);
	PTG_OUTPUT_STRING(f, ");");
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "for ($i = 0; $i < count($values); $i++) {");
	IndentIncr(f);
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "echo \"<option> $values[$i] </option>\";");
	IndentDecr(f);
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "}");
	IndentDecr(f);
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "?>");
	IndentNewLine(f);
}

#ifdef PROTO_OK
PTGNode PTGFormularConstantSelectOption(PTGNode p1)
#else
PTGNode PTGFormularConstantSelectOption(p1)
PTGNode p1;
#endif
{
	_PPTGFormularConstantSelectOption n;

	n = (_PPTGFormularConstantSelectOption)MALLOC(sizeof(struct _SPTGFormularConstantSelectOption));
	n->_print = (_PTGProc)_PrPTGFormularConstantSelectOption;
	n->p1 = p1;
	return (PTGNode)n;
}

/* Implementation of Pattern FormularDataSourceSelectOption */

typedef struct _SPTGFormularDataSourceSelectOption{
	_PTGProc _print;
	PTGNode p1;
	PTGNode p2;
	PTGNode p3;
} * _PPTGFormularDataSourceSelectOption;

#ifdef PROTO_OK
static void _PrPTGFormularDataSourceSelectOption(_PPTGFormularDataSourceSelectOption n)
#else
static void _PrPTGFormularDataSourceSelectOption(n)
	_PPTGFormularDataSourceSelectOption n;
#endif
{
	PTG_OUTPUT_STRING(f, "<?php");
	IndentIncr(f);
	IndentNewLine(f);
	n->p2->_print(n->p2);
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "$options = ");
	n->p1->_print(n->p1);
	PTG_OUTPUT_STRING(f, "(");
	n->p3->_print(n->p3);
	PTG_OUTPUT_STRING(f, ");");
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "for ($i = 0; $i < count($options); $i++) {");
	IndentIncr(f);
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "echo \"<option> $options[$i] </option>\";");
	IndentDecr(f);
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "}");
	IndentDecr(f);
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "?>");
	IndentNewLine(f);
}

#ifdef PROTO_OK
PTGNode PTGFormularDataSourceSelectOption(PTGNode p1, PTGNode p2, PTGNode p3)
#else
PTGNode PTGFormularDataSourceSelectOption(p1, p2, p3)
PTGNode p1;
PTGNode p2;
PTGNode p3;
#endif
{
	_PPTGFormularDataSourceSelectOption n;

	n = (_PPTGFormularDataSourceSelectOption)MALLOC(sizeof(struct _SPTGFormularDataSourceSelectOption));
	n->_print = (_PTGProc)_PrPTGFormularDataSourceSelectOption;
	n->p1 = p1;
	n->p2 = p2;
	n->p3 = p3;
	return (PTGNode)n;
}

/* Implementation of Pattern Image */

typedef struct _SPTGImage{
	_PTGProc _print;
	PTGNode p1;
	PTGNode p2;
	PTGNode p3;
	PTGNode p4;
	PTGNode p5;
	PTGNode p6;
} * _PPTGImage;

#ifdef PROTO_OK
static void _PrPTGImage(_PPTGImage n)
#else
static void _PrPTGImage(n)
	_PPTGImage n;
#endif
{
	PTG_OUTPUT_STRING(f, "<img class='");
	n->p6->_print(n->p6);
	PTG_OUTPUT_STRING(f, "' src='");
	n->p2->_print(n->p2);
	PTG_OUTPUT_STRING(f, "' name='");
	n->p1->_print(n->p1);
	PTG_OUTPUT_STRING(f, "' width='");
	n->p4->_print(n->p4);
	PTG_OUTPUT_STRING(f, "' height='");
	n->p5->_print(n->p5);
	PTG_OUTPUT_STRING(f, "' alt='");
	n->p3->_print(n->p3);
	PTG_OUTPUT_STRING(f, "'></img>");
	IndentNewLine(f);
}

#ifdef PROTO_OK
PTGNode PTGImage(PTGNode p1, PTGNode p2, PTGNode p3, PTGNode p4, PTGNode p5, PTGNode p6)
#else
PTGNode PTGImage(p1, p2, p3, p4, p5, p6)
PTGNode p1;
PTGNode p2;
PTGNode p3;
PTGNode p4;
PTGNode p5;
PTGNode p6;
#endif
{
	_PPTGImage n;

	n = (_PPTGImage)MALLOC(sizeof(struct _SPTGImage));
	n->_print = (_PTGProc)_PrPTGImage;
	n->p1 = p1;
	n->p2 = p2;
	n->p3 = p3;
	n->p4 = p4;
	n->p5 = p5;
	n->p6 = p6;
	return (PTGNode)n;
}

/* Implementation of Pattern ImageFragment */

typedef struct _SPTGImageFragment{
	_PTGProc _print;
	PTGNode p1;
	PTGNode p2;
	PTGNode p3;
	PTGNode p4;
} * _PPTGImageFragment;

#ifdef PROTO_OK
static void _PrPTGImageFragment(_PPTGImageFragment n)
#else
static void _PrPTGImageFragment(n)
	_PPTGImageFragment n;
#endif
{
	PTG_OUTPUT_STRING(f, "<img src='");
	n->p1->_print(n->p1);
	PTG_OUTPUT_STRING(f, "' width='");
	n->p3->_print(n->p3);
	PTG_OUTPUT_STRING(f, "' height='");
	n->p4->_print(n->p4);
	PTG_OUTPUT_STRING(f, "' alt='");
	n->p2->_print(n->p2);
	PTG_OUTPUT_STRING(f, "'></img>");
	IndentNewLine(f);
}

#ifdef PROTO_OK
PTGNode PTGImageFragment(PTGNode p1, PTGNode p2, PTGNode p3, PTGNode p4)
#else
PTGNode PTGImageFragment(p1, p2, p3, p4)
PTGNode p1;
PTGNode p2;
PTGNode p3;
PTGNode p4;
#endif
{
	_PPTGImageFragment n;

	n = (_PPTGImageFragment)MALLOC(sizeof(struct _SPTGImageFragment));
	n->_print = (_PTGProc)_PrPTGImageFragment;
	n->p1 = p1;
	n->p2 = p2;
	n->p3 = p3;
	n->p4 = p4;
	return (PTGNode)n;
}

/* Implementation of Pattern Paragraph */

typedef struct _SPTGParagraph{
	_PTGProc _print;
	PTGNode p1;
	PTGNode p2;
} * _PPTGParagraph;

#ifdef PROTO_OK
static void _PrPTGParagraph(_PPTGParagraph n)
#else
static void _PrPTGParagraph(n)
	_PPTGParagraph n;
#endif
{
	PTG_OUTPUT_STRING(f, "<p class='");
	n->p1->_print(n->p1);
	PTG_OUTPUT_STRING(f, "'>");
	n->p2->_print(n->p2);
	PTG_OUTPUT_STRING(f, "</p>");
	IndentNewLine(f);
}

#ifdef PROTO_OK
PTGNode PTGParagraph(PTGNode p1, PTGNode p2)
#else
PTGNode PTGParagraph(p1, p2)
PTGNode p1;
PTGNode p2;
#endif
{
	_PPTGParagraph n;

	n = (_PPTGParagraph)MALLOC(sizeof(struct _SPTGParagraph));
	n->_print = (_PTGProc)_PrPTGParagraph;
	n->p1 = p1;
	n->p2 = p2;
	return (PTGNode)n;
}

/* No Implementation of Pattern TextFragment needed */
/* See Paragraph for implementation */

/* Implementation of Pattern Heading */

typedef struct _SPTGHeading{
	_PTGProc _print;
	PTGNode p1;
	PTGNode p2;
	PTGNode p3;
	PTGNode p4;
} * _PPTGHeading;

#ifdef PROTO_OK
static void _PrPTGHeading(_PPTGHeading n)
#else
static void _PrPTGHeading(n)
	_PPTGHeading n;
#endif
{
	PTG_OUTPUT_STRING(f, "<h");
	n->p2->_print(n->p2);
	PTG_OUTPUT_STRING(f, " class='");
	n->p1->_print(n->p1);
	PTG_OUTPUT_STRING(f, "' align='");
	n->p3->_print(n->p3);
	PTG_OUTPUT_STRING(f, "'>");
	n->p4->_print(n->p4);
	PTG_OUTPUT_STRING(f, "</h");
	n->p2->_print(n->p2);
	PTG_OUTPUT_STRING(f, ">");
	IndentNewLine(f);
}

#ifdef PROTO_OK
PTGNode PTGHeading(PTGNode p1, PTGNode p2, PTGNode p3, PTGNode p4)
#else
PTGNode PTGHeading(p1, p2, p3, p4)
PTGNode p1;
PTGNode p2;
PTGNode p3;
PTGNode p4;
#endif
{
	_PPTGHeading n;

	n = (_PPTGHeading)MALLOC(sizeof(struct _SPTGHeading));
	n->_print = (_PTGProc)_PrPTGHeading;
	n->p1 = p1;
	n->p2 = p2;
	n->p3 = p3;
	n->p4 = p4;
	return (PTGNode)n;
}

/* Implementation of Pattern HrSeq */

typedef struct _SPTGHrSeq{
	_PTGProc _print;
	PTGNode p1;
	PTGNode p2;
} * _PPTGHrSeq;

#ifdef PROTO_OK
static void _PrPTGHrSeq(_PPTGHrSeq n)
#else
static void _PrPTGHrSeq(n)
	_PPTGHrSeq n;
#endif
{
	n->p1->_print(n->p1);
	if (n->p1 != PTGNULL && n->p2 != PTGNULL)
	{
		IndentNewLine(f);
		IndentNewLine(f);
		PTG_OUTPUT_STRING(f, "<hr>");
		IndentNewLine(f);
		IndentNewLine(f);
	}
	n->p2->_print(n->p2);
}

#ifdef PROTO_OK
PTGNode PTGHrSeq(PTGNode p1, PTGNode p2)
#else
PTGNode PTGHrSeq(p1, p2)
PTGNode p1;
PTGNode p2;
#endif
{
	_PPTGHrSeq n;

	n = (_PPTGHrSeq)MALLOC(sizeof(struct _SPTGHrSeq));
	if(p1 == PTGNULL && p2 == PTGNULL)
		return PTGNULL;

	n->_print = (_PTGProc)_PrPTGHrSeq;
	n->p1 = p1;
	n->p2 = p2;
	return (PTGNode)n;
}

/* No Implementation of Pattern MetaInformation needed */
/* See FormularRadioGroupElement for implementation */

/* Implementation of Pattern MetaInformationElement */

typedef struct _SPTGMetaInformationElement{
	_PTGProc _print;
	PTGNode p1;
	PTGNode p2;
} * _PPTGMetaInformationElement;

#ifdef PROTO_OK
static void _PrPTGMetaInformationElement(_PPTGMetaInformationElement n)
#else
static void _PrPTGMetaInformationElement(n)
	_PPTGMetaInformationElement n;
#endif
{
	PTG_OUTPUT_STRING(f, "<meta name=\"");
	n->p1->_print(n->p1);
	PTG_OUTPUT_STRING(f, "\" content=\"");
	n->p2->_print(n->p2);
	PTG_OUTPUT_STRING(f, "\"/>");
}

#ifdef PROTO_OK
PTGNode PTGMetaInformationElement(PTGNode p1, PTGNode p2)
#else
PTGNode PTGMetaInformationElement(p1, p2)
PTGNode p1;
PTGNode p2;
#endif
{
	_PPTGMetaInformationElement n;

	n = (_PPTGMetaInformationElement)MALLOC(sizeof(struct _SPTGMetaInformationElement));
	n->_print = (_PTGProc)_PrPTGMetaInformationElement;
	n->p1 = p1;
	n->p2 = p2;
	return (PTGNode)n;
}

/* No Implementation of Pattern StyleSheet needed */
/* See MetaInformation for implementation */

/* Implementation of Pattern StyleSheetBlock */

typedef struct _SPTGStyleSheetBlock{
	_PTGProc _print;
	PTGNode p1;
	PTGNode p2;
} * _PPTGStyleSheetBlock;

#ifdef PROTO_OK
static void _PrPTGStyleSheetBlock(_PPTGStyleSheetBlock n)
#else
static void _PrPTGStyleSheetBlock(n)
	_PPTGStyleSheetBlock n;
#endif
{
	n->p1->_print(n->p1);
	PTG_OUTPUT_STRING(f, " {");
	n->p2->_print(n->p2);
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "}");
	IndentNewLine(f);
}

#ifdef PROTO_OK
PTGNode PTGStyleSheetBlock(PTGNode p1, PTGNode p2)
#else
PTGNode PTGStyleSheetBlock(p1, p2)
PTGNode p1;
PTGNode p2;
#endif
{
	_PPTGStyleSheetBlock n;

	n = (_PPTGStyleSheetBlock)MALLOC(sizeof(struct _SPTGStyleSheetBlock));
	n->_print = (_PTGProc)_PrPTGStyleSheetBlock;
	n->p1 = p1;
	n->p2 = p2;
	return (PTGNode)n;
}

/* Implementation of Pattern StyleSheetPair */

typedef struct _SPTGStyleSheetPair{
	_PTGProc _print;
	PTGNode p1;
	PTGNode p2;
	PTGNode p3;
} * _PPTGStyleSheetPair;

#ifdef PROTO_OK
static void _PrPTGStyleSheetPair(_PPTGStyleSheetPair n)
#else
static void _PrPTGStyleSheetPair(n)
	_PPTGStyleSheetPair n;
#endif
{
	PTG_OUTPUT_STRING(f, "/*");
	n->p2->_print(n->p2);
	PTG_OUTPUT_STRING(f, " Attributes */");
	IndentNewLine(f);
	n->p1->_print(n->p1);
	PTG_OUTPUT_STRING(f, ":");
	n->p3->_print(n->p3);
	PTG_OUTPUT_STRING(f, ";");
}

#ifdef PROTO_OK
PTGNode PTGStyleSheetPair(PTGNode p1, PTGNode p2, PTGNode p3)
#else
PTGNode PTGStyleSheetPair(p1, p2, p3)
PTGNode p1;
PTGNode p2;
PTGNode p3;
#endif
{
	_PPTGStyleSheetPair n;

	n = (_PPTGStyleSheetPair)MALLOC(sizeof(struct _SPTGStyleSheetPair));
	n->_print = (_PTGProc)_PrPTGStyleSheetPair;
	n->p1 = p1;
	n->p2 = p2;
	n->p3 = p3;
	return (PTGNode)n;
}

/* Implementation of Pattern NewLine */

typedef struct _SPTGNewLine{
	_PTGProc _print;
	PTGNode p1;
	PTGNode p2;
} * _PPTGNewLine;

#ifdef PROTO_OK
static void _PrPTGNewLine(_PPTGNewLine n)
#else
static void _PrPTGNewLine(n)
	_PPTGNewLine n;
#endif
{
	n->p1->_print(n->p1);
	IndentNewLine(f);
	n->p2->_print(n->p2);
}

#ifdef PROTO_OK
PTGNode PTGNewLine(PTGNode p1, PTGNode p2)
#else
PTGNode PTGNewLine(p1, p2)
PTGNode p1;
PTGNode p2;
#endif
{
	_PPTGNewLine n;

	n = (_PPTGNewLine)MALLOC(sizeof(struct _SPTGNewLine));
	n->_print = (_PTGProc)_PrPTGNewLine;
	n->p1 = p1;
	n->p2 = p2;
	return (PTGNode)n;
}

/* Implementation of Pattern Ref */

typedef struct _SPTGRef{
	_PTGProc _print;
	PTGNode p1;
	PTGNode p2;
} * _PPTGRef;

#ifdef PROTO_OK
static void _PrPTGRef(_PPTGRef n)
#else
static void _PrPTGRef(n)
	_PPTGRef n;
#endif
{
	PTG_OUTPUT_STRING(f, "<a href='");
	n->p1->_print(n->p1);
	PTG_OUTPUT_STRING(f, "' >");
	n->p2->_print(n->p2);
	PTG_OUTPUT_STRING(f, "</a>");
}

#ifdef PROTO_OK
PTGNode PTGRef(PTGNode p1, PTGNode p2)
#else
PTGNode PTGRef(p1, p2)
PTGNode p1;
PTGNode p2;
#endif
{
	_PPTGRef n;

	n = (_PPTGRef)MALLOC(sizeof(struct _SPTGRef));
	n->_print = (_PTGProc)_PrPTGRef;
	n->p1 = p1;
	n->p2 = p2;
	return (PTGNode)n;
}

/* Implementation of Pattern Table */

typedef struct _SPTGTable{
	_PTGProc _print;
	PTGNode p1;
	PTGNode p2;
} * _PPTGTable;

#ifdef PROTO_OK
static void _PrPTGTable(_PPTGTable n)
#else
static void _PrPTGTable(n)
	_PPTGTable n;
#endif
{
	PTG_OUTPUT_STRING(f, "<table class='");
	n->p1->_print(n->p1);
	PTG_OUTPUT_STRING(f, "' border='0' cellpadding='0' cellspacing='0' >");
	IndentNewLine(f);
	n->p2->_print(n->p2);
	PTG_OUTPUT_STRING(f, "</table>");
	IndentNewLine(f);
}

#ifdef PROTO_OK
PTGNode PTGTable(PTGNode p1, PTGNode p2)
#else
PTGNode PTGTable(p1, p2)
PTGNode p1;
PTGNode p2;
#endif
{
	_PPTGTable n;

	n = (_PPTGTable)MALLOC(sizeof(struct _SPTGTable));
	n->_print = (_PTGProc)_PrPTGTable;
	n->p1 = p1;
	n->p2 = p2;
	return (PTGNode)n;
}

/* Implementation of Pattern TableRow */

typedef struct _SPTGTableRow{
	_PTGProc _print;
	PTGNode p1;
} * _PPTGTableRow;

#ifdef PROTO_OK
static void _PrPTGTableRow(_PPTGTableRow n)
#else
static void _PrPTGTableRow(n)
	_PPTGTableRow n;
#endif
{
	PTG_OUTPUT_STRING(f, "<tr>");
	IndentNewLine(f);
	n->p1->_print(n->p1);
	PTG_OUTPUT_STRING(f, "</tr>");
	IndentNewLine(f);
}

#ifdef PROTO_OK
PTGNode PTGTableRow(PTGNode p1)
#else
PTGNode PTGTableRow(p1)
PTGNode p1;
#endif
{
	_PPTGTableRow n;

	n = (_PPTGTableRow)MALLOC(sizeof(struct _SPTGTableRow));
	n->_print = (_PTGProc)_PrPTGTableRow;
	n->p1 = p1;
	return (PTGNode)n;
}

/* Implementation of Pattern TableCell */

typedef struct _SPTGTableCell{
	_PTGProc _print;
	PTGNode p1;
} * _PPTGTableCell;

#ifdef PROTO_OK
static void _PrPTGTableCell(_PPTGTableCell n)
#else
static void _PrPTGTableCell(n)
	_PPTGTableCell n;
#endif
{
	PTG_OUTPUT_STRING(f, "<td>");
	IndentNewLine(f);
	n->p1->_print(n->p1);
	PTG_OUTPUT_STRING(f, "</td>");
	IndentNewLine(f);
}

#ifdef PROTO_OK
PTGNode PTGTableCell(PTGNode p1)
#else
PTGNode PTGTableCell(p1)
PTGNode p1;
#endif
{
	_PPTGTableCell n;

	n = (_PPTGTableCell)MALLOC(sizeof(struct _SPTGTableCell));
	n->_print = (_PTGProc)_PrPTGTableCell;
	n->p1 = p1;
	return (PTGNode)n;
}

/* Implementation of Pattern OutputElement */

typedef struct _SPTGOutputElement{
	_PTGProc _print;
	PTGNode p1;
} * _PPTGOutputElement;

#ifdef PROTO_OK
static void _PrPTGOutputElement(_PPTGOutputElement n)
#else
static void _PrPTGOutputElement(n)
	_PPTGOutputElement n;
#endif
{
	PTG_OUTPUT_STRING(f, "<?php");
	IndentIncr(f);
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "echo $");
	n->p1->_print(n->p1);
	PTG_OUTPUT_STRING(f, ";");
	IndentDecr(f);
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "?>");
	IndentNewLine(f);
}

#ifdef PROTO_OK
PTGNode PTGOutputElement(PTGNode p1)
#else
PTGNode PTGOutputElement(p1)
PTGNode p1;
#endif
{
	_PPTGOutputElement n;

	n = (_PPTGOutputElement)MALLOC(sizeof(struct _SPTGOutputElement));
	n->_print = (_PTGProc)_PrPTGOutputElement;
	n->p1 = p1;
	return (PTGNode)n;
}

/* Implementation of Pattern DataSourceActualParam */

typedef struct _SPTGDataSourceActualParam{
	_PTGProc _print;
	PTGNode p1;
} * _PPTGDataSourceActualParam;

#ifdef PROTO_OK
static void _PrPTGDataSourceActualParam(_PPTGDataSourceActualParam n)
#else
static void _PrPTGDataSourceActualParam(n)
	_PPTGDataSourceActualParam n;
#endif
{
	PTG_OUTPUT_STRING(f, "$");
	n->p1->_print(n->p1);
}

#ifdef PROTO_OK
PTGNode PTGDataSourceActualParam(PTGNode p1)
#else
PTGNode PTGDataSourceActualParam(p1)
PTGNode p1;
#endif
{
	_PPTGDataSourceActualParam n;

	n = (_PPTGDataSourceActualParam)MALLOC(sizeof(struct _SPTGDataSourceActualParam));
	n->_print = (_PTGProc)_PrPTGDataSourceActualParam;
	n->p1 = p1;
	return (PTGNode)n;
}

/* Implementation of Pattern ListOutputElement */

typedef struct _SPTGListOutputElement{
	_PTGProc _print;
	PTGNode p1;
	PTGNode p2;
	PTGNode p3;
	PTGNode p4;
} * _PPTGListOutputElement;

#ifdef PROTO_OK
static void _PrPTGListOutputElement(_PPTGListOutputElement n)
#else
static void _PrPTGListOutputElement(n)
	_PPTGListOutputElement n;
#endif
{
	PTG_OUTPUT_STRING(f, "<?php");
	IndentIncr(f);
	IndentNewLine(f);
	n->p1->_print(n->p1);
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "$result=");
	n->p3->_print(n->p3);
	PTG_OUTPUT_STRING(f, "(");
	n->p2->_print(n->p2);
	PTG_OUTPUT_STRING(f, ");");
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "echo \"<table class='");
	n->p4->_print(n->p4);
	PTG_OUTPUT_STRING(f, "'>\";");
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "for ($i=0; $i < mysql_num_fields($result); $i++) //Table Header");
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "{ print \"<th>\".mysql_field_name($result, $i).\"</th>\"; }");
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "while(@$row = mysql_fetch_row($result))");
	IndentIncr(f);
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "{");
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "echo   \"<tr>\n\";");
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "$i=0;");
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "while($i < mysql_num_fields($result))");
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "{");
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "echo   \"<td>$row[$i]</td>\n\";");
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "$i++;");
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "}");
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "echo   \"</tr>\n\";");
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "}");
	IndentDecr(f);
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "echo   \"</table>\n\";");
	IndentNewLine(f);
	IndentDecr(f);
	PTG_OUTPUT_STRING(f, "?>");
	IndentNewLine(f);
}

#ifdef PROTO_OK
PTGNode PTGListOutputElement(PTGNode p1, PTGNode p2, PTGNode p3, PTGNode p4)
#else
PTGNode PTGListOutputElement(p1, p2, p3, p4)
PTGNode p1;
PTGNode p2;
PTGNode p3;
PTGNode p4;
#endif
{
	_PPTGListOutputElement n;

	n = (_PPTGListOutputElement)MALLOC(sizeof(struct _SPTGListOutputElement));
	n->_print = (_PTGProc)_PrPTGListOutputElement;
	n->p1 = p1;
	n->p2 = p2;
	n->p3 = p3;
	n->p4 = p4;
	return (PTGNode)n;
}

/* Implementation of Pattern ListOutputElementWithLimitClause */

typedef struct _SPTGListOutputElementWithLimitClause{
	_PTGProc _print;
	PTGNode p1;
	PTGNode p3;
	PTGNode p4;
	PTGNode p5;
	PTGNode p6;
} * _PPTGListOutputElementWithLimitClause;

#ifdef PROTO_OK
static void _PrPTGListOutputElementWithLimitClause(_PPTGListOutputElementWithLimitClause n)
#else
static void _PrPTGListOutputElementWithLimitClause(n)
	_PPTGListOutputElementWithLimitClause n;
#endif
{
	PTG_OUTPUT_STRING(f, "<?php");
	IndentIncr(f);
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "$maxElements = ");
	n->p6->_print(n->p6);
	PTG_OUTPUT_STRING(f, ";\n");
	PTG_OUTPUT_STRING(f, "$offset=0\n;");
	PTG_OUTPUT_STRING(f, "if (!isset($_REQUEST['offset'])) {\n");
	PTG_OUTPUT_STRING(f, "  $offset=$_REQUEST['offset'];\n");
	PTG_OUTPUT_STRING(f, " }\n");
	n->p1->_print(n->p1);
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "$result=");
	n->p3->_print(n->p3);
	PTG_OUTPUT_STRING(f, "($offset,($offset+$maxElements));");
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "echo \"<table>\";");
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "while(@$row = mysql_fetch_row($result))");
	IndentIncr(f);
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "{");
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "echo   \"<tr>\n\";");
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "$i=0;");
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "while($i < mysql_num_fields($result))");
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "{");
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "echo   \"<td>$row[$i]</td>\n\";");
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "$i++;");
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "}");
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "echo   \"</tr>\n\";");
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "}");
	IndentDecr(f);
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "echo   \"</table>\n\";");
	IndentNewLine(f);
	IndentDecr(f);
	PTG_OUTPUT_STRING(f, "$website=\"");
	n->p4->_print(n->p4);
	PTG_OUTPUT_STRING(f, "\";\n");
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "$back=$offset-$maxElements;\n");
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "$forward=$offset+$maxElements;\n");
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "echo \"<div class='listNavigation'>\";");
	n->p5->_print(n->p5);
	PTG_OUTPUT_STRING(f, "echo \"</div> \";");
	PTG_OUTPUT_STRING(f, "?>");
	IndentNewLine(f);
}

#ifdef PROTO_OK
PTGNode PTGListOutputElementWithLimitClause(PTGNode p1, PTGNode p2, PTGNode p3, PTGNode p4, PTGNode p5, PTGNode p6)
#else
PTGNode PTGListOutputElementWithLimitClause(p1, p2, p3, p4, p5, p6)
PTGNode p1;
PTGNode p2;
PTGNode p3;
PTGNode p4;
PTGNode p5;
PTGNode p6;
#endif
{
	_PPTGListOutputElementWithLimitClause n;

	n = (_PPTGListOutputElementWithLimitClause)MALLOC(sizeof(struct _SPTGListOutputElementWithLimitClause));
	n->_print = (_PTGProc)_PrPTGListOutputElementWithLimitClause;
	n->p1 = p1;
	n->p3 = p3;
	n->p4 = p4;
	n->p5 = p5;
	n->p6 = p6;
	return (PTGNode)n;
}

/* Implementation of Pattern WebNodeActualParam */

typedef struct _SPTGWebNodeActualParam{
	_PTGProc _print;
	PTGNode p1;
	PTGNode p2;
} * _PPTGWebNodeActualParam;

#ifdef PROTO_OK
static void _PrPTGWebNodeActualParam(_PPTGWebNodeActualParam n)
#else
static void _PrPTGWebNodeActualParam(n)
	_PPTGWebNodeActualParam n;
#endif
{
	PTG_OUTPUT_STRING(f, "$");
	n->p1->_print(n->p1);
	PTG_OUTPUT_STRING(f, " = ");
	n->p2->_print(n->p2);
	PTG_OUTPUT_STRING(f, ";");
	IndentNewLine(f);
}

#ifdef PROTO_OK
PTGNode PTGWebNodeActualParam(PTGNode p1, PTGNode p2)
#else
PTGNode PTGWebNodeActualParam(p1, p2)
PTGNode p1;
PTGNode p2;
#endif
{
	_PPTGWebNodeActualParam n;

	n = (_PPTGWebNodeActualParam)MALLOC(sizeof(struct _SPTGWebNodeActualParam));
	n->_print = (_PTGProc)_PrPTGWebNodeActualParam;
	n->p1 = p1;
	n->p2 = p2;
	return (PTGNode)n;
}

/* No Implementation of Pattern WebNodeVariableExpression needed */
/* See FieldValue for implementation */

/* Implementation of Pattern WebNodeVariableExpression2 */

typedef struct _SPTGWebNodeVariableExpression2{
	_PTGProc _print;
	PTGNode p1;
} * _PPTGWebNodeVariableExpression2;

#ifdef PROTO_OK
static void _PrPTGWebNodeVariableExpression2(_PPTGWebNodeVariableExpression2 n)
#else
static void _PrPTGWebNodeVariableExpression2(n)
	_PPTGWebNodeVariableExpression2 n;
#endif
{
	n->p1->_print(n->p1);
}

#ifdef PROTO_OK
PTGNode PTGWebNodeVariableExpression2(PTGNode p1)
#else
PTGNode PTGWebNodeVariableExpression2(p1)
PTGNode p1;
#endif
{
	_PPTGWebNodeVariableExpression2 n;

	n = (_PPTGWebNodeVariableExpression2)MALLOC(sizeof(struct _SPTGWebNodeVariableExpression2));
	if(p1 == PTGNULL)
		return PTGNULL;

	n->_print = (_PTGProc)_PrPTGWebNodeVariableExpression2;
	n->p1 = p1;
	return (PTGNode)n;
}

/* Implementation of Pattern NavigationBar */

typedef struct _SPTGNavigationBar{
	_PTGProc _print;
	PTGNode p1;
	PTGNode p2;
	PTGNode p3;
} * _PPTGNavigationBar;

#ifdef PROTO_OK
static void _PrPTGNavigationBar(_PPTGNavigationBar n)
#else
static void _PrPTGNavigationBar(n)
	_PPTGNavigationBar n;
#endif
{
	PTG_OUTPUT_STRING(f, "<?php ");
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "$navigationbar_alignment=\"");
	n->p2->_print(n->p2);
	PTG_OUTPUT_STRING(f, "\";");
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "if (!$navigationbar_alignment){ $navigationbar_alignment='left'; };");
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "$navigationbar_textwrap=\"wrap\";");
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "$navigationbar_display=\"$navigationbar_alignment\";");
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "if (!($navigationbar_alignment == \"left\" || $navigationbar_alignment == \"right\")){");
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "	$navigationbar_textwrap =\"nowrap\";");
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "       $navigationbar_display =\"inline\";");
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "}");
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "echo \"<ul class=\\\"navigationbar\\\" style=\\\"float: $navigationbar_alignment; white-space: $navigationbar_textwrap\\\">\";");
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "$navelementsarr 	= array();");
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "$navelementsarrname 	= array();");
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "$navelementsarrwebzone = array();");
	IndentNewLine(f);
	n->p1->_print(n->p1);
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "for($i=0;$i<sizeof($navelementsarr);$i++){");
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "if (webZoneAllowed($navelementsarruserlist[$i])){");
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "echo \"<li style=\\\"display: $navigationbar_display\\\"><a class=\\\"");
	n->p3->_print(n->p3);
	PTG_OUTPUT_STRING(f, "\\\" href=\\\"$navelementsarr[$i]\\\">$navelementsarrname[$i]</a></li>\";");
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "} ");
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "} ");
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "echo \"</ul><br clear='all'></br>\n\";");
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "function webZoneAllowed($permission_list)");
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "{");
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "$web_rollen = explode(' ', $permission_list);");
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "$ben_rollen = explode(' ', $_SESSION['Rollen_liste']);");
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "if ((strlen($permission_list) > 0) && (!isset($_SESSION['user_nickname'])))");
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "{");
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "return false;");
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "} else {");
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "if ($_SESSION['Rollen_liste'] != $permission_list && $permission_list!='')");
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "{");
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "$result = array_intersect($ben_rollen, $web_rollen);");
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "$result1 = count($result);");
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "if ($result1 != 1)");
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "{");
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "return false;");
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "}");
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "else {");
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "return true;");
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "}");
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "}");
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, " else {");
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "return true;");
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "}");
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "}");
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "}");
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "?>");
	IndentNewLine(f);
}

#ifdef PROTO_OK
PTGNode PTGNavigationBar(PTGNode p1, PTGNode p2, PTGNode p3)
#else
PTGNode PTGNavigationBar(p1, p2, p3)
PTGNode p1;
PTGNode p2;
PTGNode p3;
#endif
{
	_PPTGNavigationBar n;

	n = (_PPTGNavigationBar)MALLOC(sizeof(struct _SPTGNavigationBar));
	n->_print = (_PTGProc)_PrPTGNavigationBar;
	n->p1 = p1;
	n->p2 = p2;
	n->p3 = p3;
	return (PTGNode)n;
}

/* Implementation of Pattern NavigationBarElement */

typedef struct _SPTGNavigationBarElement{
	_PTGProc _print;
	PTGNode p1;
	char * p2;
	char * p3;
	PTGNode p4;
} * _PPTGNavigationBarElement;

#ifdef PROTO_OK
static void _PrPTGNavigationBarElement(_PPTGNavigationBarElement n)
#else
static void _PrPTGNavigationBarElement(n)
	_PPTGNavigationBarElement n;
#endif
{
	PTG_OUTPUT_STRING(f, "$navelementsarr[");
	n->p1->_print(n->p1);
	PTG_OUTPUT_STRING(f, "]=\"");
	PTG_OUTPUT_STRING(f, n->p2);
	PTG_OUTPUT_STRING(f, "\";");
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "$navelementsarrname[");
	n->p1->_print(n->p1);
	PTG_OUTPUT_STRING(f, "]=\"");
	PTG_OUTPUT_STRING(f, n->p3);
	PTG_OUTPUT_STRING(f, "\";");
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "$navelementsarruserlist[");
	n->p1->_print(n->p1);
	PTG_OUTPUT_STRING(f, "]=\"");
	n->p4->_print(n->p4);
	PTG_OUTPUT_STRING(f, "\";");
	IndentNewLine(f);
}

#ifdef PROTO_OK
PTGNode PTGNavigationBarElement(PTGNode p1, char * p2, char * p3, PTGNode p4)
#else
PTGNode PTGNavigationBarElement(p1, p2, p3, p4)
PTGNode p1;
char * p2;
char * p3;
PTGNode p4;
#endif
{
	_PPTGNavigationBarElement n;

	n = (_PPTGNavigationBarElement)MALLOC(sizeof(struct _SPTGNavigationBarElement));
	n->_print = (_PTGProc)_PrPTGNavigationBarElement;
	n->p1 = p1;
	n->p2 = p2;
	n->p3 = p3;
	n->p4 = p4;
	return (PTGNode)n;
}

/* Implementation of Pattern PHPGlobals */

typedef struct _SPTGPHPGlobals{
	_PTGProc _print;
	PTGNode p1;
} * _PPTGPHPGlobals;

#ifdef PROTO_OK
static void _PrPTGPHPGlobals(_PPTGPHPGlobals n)
#else
static void _PrPTGPHPGlobals(n)
	_PPTGPHPGlobals n;
#endif
{
	PTG_OUTPUT_STRING(f, "<?php");
	IndentNewLine(f);
	n->p1->_print(n->p1);
	PTG_OUTPUT_STRING(f, "?>");
	IndentNewLine(f);
}

#ifdef PROTO_OK
PTGNode PTGPHPGlobals(PTGNode p1)
#else
PTGNode PTGPHPGlobals(p1)
PTGNode p1;
#endif
{
	_PPTGPHPGlobals n;

	n = (_PPTGPHPGlobals)MALLOC(sizeof(struct _SPTGPHPGlobals));
	n->_print = (_PTGProc)_PrPTGPHPGlobals;
	n->p1 = p1;
	return (PTGNode)n;
}

/* Implementation of Pattern PHPArray */

typedef struct _SPTGPHPArray{
	_PTGProc _print;
	PTGNode p1;
	PTGNode p2;
} * _PPTGPHPArray;

#ifdef PROTO_OK
static void _PrPTGPHPArray(_PPTGPHPArray n)
#else
static void _PrPTGPHPArray(n)
	_PPTGPHPArray n;
#endif
{
	PTG_OUTPUT_STRING(f, "$");
	n->p1->_print(n->p1);
	PTG_OUTPUT_STRING(f, " = Array(");
	n->p2->_print(n->p2);
	PTG_OUTPUT_STRING(f, ");");
	IndentNewLine(f);
}

#ifdef PROTO_OK
PTGNode PTGPHPArray(PTGNode p1, PTGNode p2)
#else
PTGNode PTGPHPArray(p1, p2)
PTGNode p1;
PTGNode p2;
#endif
{
	_PPTGPHPArray n;

	n = (_PPTGPHPArray)MALLOC(sizeof(struct _SPTGPHPArray));
	n->_print = (_PTGProc)_PrPTGPHPArray;
	n->p1 = p1;
	n->p2 = p2;
	return (PTGNode)n;
}

/* Implementation of Pattern Apostroph */

typedef struct _SPTGApostroph{
	_PTGProc _print;
	PTGNode p1;
} * _PPTGApostroph;

#ifdef PROTO_OK
static void _PrPTGApostroph(_PPTGApostroph n)
#else
static void _PrPTGApostroph(n)
	_PPTGApostroph n;
#endif
{
	PTG_OUTPUT_STRING(f, "'");
	n->p1->_print(n->p1);
	PTG_OUTPUT_STRING(f, "'");
}

#ifdef PROTO_OK
PTGNode PTGApostroph(PTGNode p1)
#else
PTGNode PTGApostroph(p1)
PTGNode p1;
#endif
{
	_PPTGApostroph n;

	n = (_PPTGApostroph)MALLOC(sizeof(struct _SPTGApostroph));
	n->_print = (_PTGProc)_PrPTGApostroph;
	n->p1 = p1;
	return (PTGNode)n;
}

/* Implementation of Pattern DBColumn */

typedef struct _SPTGDBColumn{
	_PTGProc _print;
	PTGNode p1;
	PTGNode p2;
} * _PPTGDBColumn;

#ifdef PROTO_OK
static void _PrPTGDBColumn(_PPTGDBColumn n)
#else
static void _PrPTGDBColumn(n)
	_PPTGDBColumn n;
#endif
{
	n->p1->_print(n->p1);
	PTG_OUTPUT_STRING(f, " ");
	n->p2->_print(n->p2);
	PTG_OUTPUT_STRING(f, "\n");
}

#ifdef PROTO_OK
PTGNode PTGDBColumn(PTGNode p1, PTGNode p2)
#else
PTGNode PTGDBColumn(p1, p2)
PTGNode p1;
PTGNode p2;
#endif
{
	_PPTGDBColumn n;

	n = (_PPTGDBColumn)MALLOC(sizeof(struct _SPTGDBColumn));
	n->_print = (_PTGProc)_PrPTGDBColumn;
	n->p1 = p1;
	n->p2 = p2;
	return (PTGNode)n;
}

/* Implementation of Pattern DataSourceDB */

typedef struct _SPTGDataSourceDB{
	_PTGProc _print;
	PTGNode p1;
	PTGNode p2;
} * _PPTGDataSourceDB;

#ifdef PROTO_OK
static void _PrPTGDataSourceDB(_PPTGDataSourceDB n)
#else
static void _PrPTGDataSourceDB(n)
	_PPTGDataSourceDB n;
#endif
{
	PTG_OUTPUT_STRING(f, "DROP TABLE IF EXISTS ");
	n->p1->_print(n->p1);
	PTG_OUTPUT_STRING(f, ";");
	PTG_OUTPUT_STRING(f, "CREATE TABLE ");
	n->p1->_print(n->p1);
	PTG_OUTPUT_STRING(f, "(");
	n->p2->_print(n->p2);
	PTG_OUTPUT_STRING(f, ")");
}

#ifdef PROTO_OK
PTGNode PTGDataSourceDB(PTGNode p1, PTGNode p2)
#else
PTGNode PTGDataSourceDB(p1, p2)
PTGNode p1;
PTGNode p2;
#endif
{
	_PPTGDataSourceDB n;

	n = (_PPTGDataSourceDB)MALLOC(sizeof(struct _SPTGDataSourceDB));
	n->_print = (_PTGProc)_PrPTGDataSourceDB;
	n->p1 = p1;
	n->p2 = p2;
	return (PTGNode)n;
}

/* Implementation of Pattern DBFrame */

typedef struct _SPTGDBFrame{
	_PTGProc _print;
	PTGNode p1;
	PTGNode p2;
	PTGNode p3;
	PTGNode p4;
	PTGNode p5;
	PTGNode p6;
	PTGNode p7;
} * _PPTGDBFrame;

#ifdef PROTO_OK
static void _PrPTGDBFrame(_PPTGDBFrame n)
#else
static void _PrPTGDBFrame(n)
	_PPTGDBFrame n;
#endif
{
	PTG_OUTPUT_STRING(f, "<?php ");
	PTG_OUTPUT_STRING(f, "// Stelle Verbindung her mit Datenbank: ");
	n->p1->_print(n->p1);
	PTG_OUTPUT_STRING(f, "\n");
	PTG_OUTPUT_STRING(f, "// auf Host: ");
	n->p2->_print(n->p2);
	PTG_OUTPUT_STRING(f, ":");
	n->p3->_print(n->p3);
	PTG_OUTPUT_STRING(f, "\n");
	PTG_OUTPUT_STRING(f, "// Datenbanktyp: ");
	n->p4->_print(n->p4);
	PTG_OUTPUT_STRING(f, "\n");
	PTG_OUTPUT_STRING(f, "// Benutzer: ");
	n->p5->_print(n->p5);
	PTG_OUTPUT_STRING(f, "\n");
	PTG_OUTPUT_STRING(f, "// Passwort: ");
	n->p6->_print(n->p6);
	PTG_OUTPUT_STRING(f, "\n");
	PTG_OUTPUT_STRING(f, "function connect(){");
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "$host=\"");
	n->p2->_print(n->p2);
	PTG_OUTPUT_STRING(f, "\";");
	IndentIncr(f);
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "$user=\"");
	n->p5->_print(n->p5);
	PTG_OUTPUT_STRING(f, "\";");
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "$password=\"");
	n->p6->_print(n->p6);
	PTG_OUTPUT_STRING(f, "\";");
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "$databaseName=\"");
	n->p1->_print(n->p1);
	PTG_OUTPUT_STRING(f, "\";");
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "$link = mysql_pconnect($host, $user, $password);");
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "if (! $link){");
	IndentIncr(f);
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "echo \"DB Verbindung fehlgeschlagen! Versuchen Sie es spter nochmal\";");
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "echo \"Kein Zugriff auf '$host'/'$databaseName' mit: '$user', '$password'\";");
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "exit;");
	IndentDecr(f);
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "}");
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "$res = mysql_select_db($databaseName, $link)");
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "or die(\"Kein Zugriff auf '$host'/'$databaseName' mit: '$user', '$password'\");");
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "return $link;");
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "}");
	IndentDecr(f);
	IndentNewLine(f);
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "function createDBScheme(){");
	IndentIncr(f);
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "$host=\"");
	n->p2->_print(n->p2);
	PTG_OUTPUT_STRING(f, "\";");
	IndentIncr(f);
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "$user=\"");
	n->p5->_print(n->p5);
	PTG_OUTPUT_STRING(f, "\";");
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "$password=\"");
	n->p6->_print(n->p6);
	PTG_OUTPUT_STRING(f, "\";");
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "$databaseName=\"");
	n->p1->_print(n->p1);
	PTG_OUTPUT_STRING(f, "\";");
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "$link = connect();");
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, " $query =\"");
	n->p7->_print(n->p7);
	PTG_OUTPUT_STRING(f, "\";");
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, " $result = mysql_exec_batch($query, $link, true) or die(\"Anfrage fehlgeschlagen: \" . mysql_error());");
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, " if (!$result){");
	IndentNewLine(f);
	IndentIncr(f);
	PTG_OUTPUT_STRING(f, " 	echo \"Konnte Datenbank Schema nicht erstellen, bitte berprfen Sie Ihre Angaben.\";");
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "	return 0;");
	IndentDecr(f);
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, " }else{");
	IndentIncr(f);
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "echo \"Datenbankschema erfolgreich erstellt.\";");
	IndentNewLine(f);
	IndentDecr(f);
	PTG_OUTPUT_STRING(f, "}");
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "}");
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "function mysql_exec_batch ($p_query, $link, $p_transaction_safe = true) {");
	PTG_OUTPUT_STRING(f, "if ($p_transaction_safe) {");
	PTG_OUTPUT_STRING(f, " $p_query = 'START TRANSACTION;' . $p_query . '; COMMIT;';");
	PTG_OUTPUT_STRING(f, " };");
	PTG_OUTPUT_STRING(f, " $query_split = preg_split (\"/[;]+/\", $p_query);");
	PTG_OUTPUT_STRING(f, " foreach ($query_split as $command_line) {");
	PTG_OUTPUT_STRING(f, " $command_line = trim($command_line);");
	PTG_OUTPUT_STRING(f, " if ($command_line != '') {");
	PTG_OUTPUT_STRING(f, "   $query_result = mysql_query($command_line,$link);");
	PTG_OUTPUT_STRING(f, "   if ($query_result == 0) {");
	PTG_OUTPUT_STRING(f, "     break;");
	PTG_OUTPUT_STRING(f, "   };");
	PTG_OUTPUT_STRING(f, " };");
	PTG_OUTPUT_STRING(f, "};");
	PTG_OUTPUT_STRING(f, " return $query_result;");
	PTG_OUTPUT_STRING(f, "}");
	PTG_OUTPUT_STRING(f, "$action = $_REQUEST['action'];");
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "if($action==\"create\")");
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "createDBScheme();");
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "else{");
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "echo \"<html>\";");
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "echo \"<head>\";");
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "echo \"</head>\";");
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "echo \"<body>\";");
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "echo \"<form action='dbinit.php?action=create' method=post>\";");
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "echo \"<input type=submit value='Datenbank Schema anlegen'>\";");
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "echo \"</form>\";");
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "echo \"</body>\";");
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "echo \"</html>\";");
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "}");
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "?>");
}

#ifdef PROTO_OK
PTGNode PTGDBFrame(PTGNode p1, PTGNode p2, PTGNode p3, PTGNode p4, PTGNode p5, PTGNode p6, PTGNode p7)
#else
PTGNode PTGDBFrame(p1, p2, p3, p4, p5, p6, p7)
PTGNode p1;
PTGNode p2;
PTGNode p3;
PTGNode p4;
PTGNode p5;
PTGNode p6;
PTGNode p7;
#endif
{
	_PPTGDBFrame n;

	n = (_PPTGDBFrame)MALLOC(sizeof(struct _SPTGDBFrame));
	n->_print = (_PTGProc)_PrPTGDBFrame;
	n->p1 = p1;
	n->p2 = p2;
	n->p3 = p3;
	n->p4 = p4;
	n->p5 = p5;
	n->p6 = p6;
	n->p7 = p7;
	return (PTGNode)n;
}

/* Implementation of Pattern DBConnect */

typedef struct _SPTGDBConnect{
	_PTGProc _print;
	PTGNode p1;
	PTGNode p2;
	PTGNode p3;
	PTGNode p4;
	PTGNode p5;
	PTGNode p6;
} * _PPTGDBConnect;

#ifdef PROTO_OK
static void _PrPTGDBConnect(_PPTGDBConnect n)
#else
static void _PrPTGDBConnect(n)
	_PPTGDBConnect n;
#endif
{
	PTG_OUTPUT_STRING(f, "<?php ");
	PTG_OUTPUT_STRING(f, "// Stelle Verbindung her mit Datenbank: ");
	n->p1->_print(n->p1);
	PTG_OUTPUT_STRING(f, "\n");
	PTG_OUTPUT_STRING(f, "// auf Host: ");
	n->p2->_print(n->p2);
	PTG_OUTPUT_STRING(f, ":");
	n->p3->_print(n->p3);
	PTG_OUTPUT_STRING(f, "\n");
	PTG_OUTPUT_STRING(f, "// Datenbanktyp: ");
	n->p4->_print(n->p4);
	PTG_OUTPUT_STRING(f, "\n");
	PTG_OUTPUT_STRING(f, "// Benutzer: ");
	n->p5->_print(n->p5);
	PTG_OUTPUT_STRING(f, "\n");
	PTG_OUTPUT_STRING(f, "// Passwort: ");
	n->p6->_print(n->p6);
	PTG_OUTPUT_STRING(f, "\n");
	PTG_OUTPUT_STRING(f, "function connect(){");
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "$host=\"");
	n->p2->_print(n->p2);
	PTG_OUTPUT_STRING(f, "\";");
	IndentIncr(f);
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "$user=\"");
	n->p5->_print(n->p5);
	PTG_OUTPUT_STRING(f, "\";");
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "$password=\"");
	n->p6->_print(n->p6);
	PTG_OUTPUT_STRING(f, "\";");
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "$databaseName=\"");
	n->p1->_print(n->p1);
	PTG_OUTPUT_STRING(f, "\";");
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "$link = mysql_pconnect($host, $user, $password);");
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "if (! $link){");
	IndentIncr(f);
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "trigger_error(\"DB Verbindung fehlgeschlagen! Versuchen Sie es spter nochmal\", E_USER_ERROR);");
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "exit;");
	IndentDecr(f);
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "}");
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "$res = mysql_select_db($databaseName, $link);");
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "if (! $res) {");
	IndentIncr(f);
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "trigger_error(\"Kein Zugriff auf '$host'/'$databaseName' mit: '$user', '$password'\", E_USER_ERROR);");
	IndentDecr(f);
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "}");
	IndentDecr(f);
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "return $link;");
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "}");
	IndentDecr(f);
	IndentNewLine(f);
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "$link = connect();");
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "?>");
}

#ifdef PROTO_OK
PTGNode PTGDBConnect(PTGNode p1, PTGNode p2, PTGNode p3, PTGNode p4, PTGNode p5, PTGNode p6)
#else
PTGNode PTGDBConnect(p1, p2, p3, p4, p5, p6)
PTGNode p1;
PTGNode p2;
PTGNode p3;
PTGNode p4;
PTGNode p5;
PTGNode p6;
#endif
{
	_PPTGDBConnect n;

	n = (_PPTGDBConnect)MALLOC(sizeof(struct _SPTGDBConnect));
	n->_print = (_PTGProc)_PrPTGDBConnect;
	n->p1 = p1;
	n->p2 = p2;
	n->p3 = p3;
	n->p4 = p4;
	n->p5 = p5;
	n->p6 = p6;
	return (PTGNode)n;
}

/* Implementation of Pattern BatchSeparator */

typedef struct _SPTGBatchSeparator{
	_PTGProc _print;
	PTGNode p1;
	PTGNode p2;
} * _PPTGBatchSeparator;

#ifdef PROTO_OK
static void _PrPTGBatchSeparator(_PPTGBatchSeparator n)
#else
static void _PrPTGBatchSeparator(n)
	_PPTGBatchSeparator n;
#endif
{
	n->p1->_print(n->p1);
	if (n->p1 != PTGNULL && n->p2 != PTGNULL)
	{
		PTG_OUTPUT_STRING(f, ";");
	}
	n->p2->_print(n->p2);
}

#ifdef PROTO_OK
PTGNode PTGBatchSeparator(PTGNode p1, PTGNode p2)
#else
PTGNode PTGBatchSeparator(p1, p2)
PTGNode p1;
PTGNode p2;
#endif
{
	_PPTGBatchSeparator n;

	n = (_PPTGBatchSeparator)MALLOC(sizeof(struct _SPTGBatchSeparator));
	if(p1 == PTGNULL && p2 == PTGNULL)
		return PTGNULL;

	n->_print = (_PTGProc)_PrPTGBatchSeparator;
	n->p1 = p1;
	n->p2 = p2;
	return (PTGNode)n;
}

/* No Implementation of Pattern DataSourceFilter needed */
/* See AsIs for implementation */

/* Implementation of Pattern DataSourceDBFunction */

typedef struct _SPTGDataSourceDBFunction{
	_PTGProc _print;
	char * p1;
} * _PPTGDataSourceDBFunction;

#ifdef PROTO_OK
static void _PrPTGDataSourceDBFunction(_PPTGDataSourceDBFunction n)
#else
static void _PrPTGDataSourceDBFunction(n)
	_PPTGDataSourceDBFunction n;
#endif
{
	PTG_OUTPUT_STRING(f, "function ");
	PTG_OUTPUT_STRING(f, n->p1);
	PTG_OUTPUT_STRING(f, "() {");
	IndentNewLine(f);
	IndentIncr(f);
	PTG_OUTPUT_STRING(f, "global $link;");
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "$result = mysql_query(\"select * from ");
	PTG_OUTPUT_STRING(f, n->p1);
	PTG_OUTPUT_STRING(f, "\",$link);");
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "if (! $result) { trigger_error(mysql_error(), E_USER_ERROR); }");
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "return $result;");
	IndentDecr(f);
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "}");
}

#ifdef PROTO_OK
PTGNode PTGDataSourceDBFunction(char * p1)
#else
PTGNode PTGDataSourceDBFunction(p1)
char * p1;
#endif
{
	_PPTGDataSourceDBFunction n;

	n = (_PPTGDataSourceDBFunction)MALLOC(sizeof(struct _SPTGDataSourceDBFunction));
	n->_print = (_PTGProc)_PrPTGDataSourceDBFunction;
	n->p1 = p1;
	return (PTGNode)n;
}

/* Implementation of Pattern DataSourceUpdate */

typedef struct _SPTGDataSourceUpdate{
	_PTGProc _print;
	PTGNode p1;
	PTGNode p2;
	char * p3;
} * _PPTGDataSourceUpdate;

#ifdef PROTO_OK
static void _PrPTGDataSourceUpdate(_PPTGDataSourceUpdate n)
#else
static void _PrPTGDataSourceUpdate(n)
	_PPTGDataSourceUpdate n;
#endif
{
	PTG_OUTPUT_STRING(f, "function ");
	PTG_OUTPUT_STRING(f, n->p3);
	PTG_OUTPUT_STRING(f, "(");
	n->p2->_print(n->p2);
	PTG_OUTPUT_STRING(f, ") {");
	IndentNewLine(f);
	IndentIncr(f);
	PTG_OUTPUT_STRING(f, "global $link;");
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "$result = mysql_query(\"");
	n->p1->_print(n->p1);
	PTG_OUTPUT_STRING(f, "\",$link);");
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "if (! $result) { trigger_error(mysql_error(), E_USER_ERROR); }");
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "return $result;");
	IndentDecr(f);
	IndentNewLine(f);
	PTG_OUTPUT_STRING(f, "}");
}

#ifdef PROTO_OK
PTGNode PTGDataSourceUpdate(PTGNode p1, PTGNode p2, char * p3)
#else
PTGNode PTGDataSourceUpdate(p1, p2, p3)
PTGNode p1;
PTGNode p2;
char * p3;
#endif
{
	_PPTGDataSourceUpdate n;

	n = (_PPTGDataSourceUpdate)MALLOC(sizeof(struct _SPTGDataSourceUpdate));
	n->_print = (_PTGProc)_PrPTGDataSourceUpdate;
	n->p1 = p1;
	n->p2 = p2;
	n->p3 = p3;
	return (PTGNode)n;
}

/* No Implementation of Pattern DataSourceInsert needed */
/* See DataSourceUpdate for implementation */

/* No Implementation of Pattern DataSourceProjectionFilter needed */
/* See DataSourceInsert for implementation */

/* No Implementation of Pattern LimitDataSource needed */
/* See DataSourceProjectionFilter for implementation */

/* No Implementation of Pattern DataSourceDelete needed */
/* See LimitDataSource for implementation */

/* Implementation of Pattern DataSourceConstantExpression */

typedef struct _SPTGDataSourceConstantExpression{
	_PTGProc _print;
	PTGNode p1;
} * _PPTGDataSourceConstantExpression;

#ifdef PROTO_OK
static void _PrPTGDataSourceConstantExpression(_PPTGDataSourceConstantExpression n)
#else
static void _PrPTGDataSourceConstantExpression(n)
	_PPTGDataSourceConstantExpression n;
#endif
{
	PTG_OUTPUT_STRING(f, "\".");
	n->p1->_print(n->p1);
	PTG_OUTPUT_STRING(f, ".\"");
}

#ifdef PROTO_OK
PTGNode PTGDataSourceConstantExpression(PTGNode p1)
#else
PTGNode PTGDataSourceConstantExpression(p1)
PTGNode p1;
#endif
{
	_PPTGDataSourceConstantExpression n;

	n = (_PPTGDataSourceConstantExpression)MALLOC(sizeof(struct _SPTGDataSourceConstantExpression));
	n->_print = (_PTGProc)_PrPTGDataSourceConstantExpression;
	n->p1 = p1;
	return (PTGNode)n;
}

/* Implementation of Pattern DataSourceVariableExpression */

typedef struct _SPTGDataSourceVariableExpression{
	_PTGProc _print;
	PTGNode p1;
} * _PPTGDataSourceVariableExpression;

#ifdef PROTO_OK
static void _PrPTGDataSourceVariableExpression(_PPTGDataSourceVariableExpression n)
#else
static void _PrPTGDataSourceVariableExpression(n)
	_PPTGDataSourceVariableExpression n;
#endif
{
	PTG_OUTPUT_STRING(f, "$_REQUEST['");
	n->p1->_print(n->p1);
	PTG_OUTPUT_STRING(f, "'];");
}

#ifdef PROTO_OK
PTGNode PTGDataSourceVariableExpression(PTGNode p1)
#else
PTGNode PTGDataSourceVariableExpression(p1)
PTGNode p1;
#endif
{
	_PPTGDataSourceVariableExpression n;

	n = (_PPTGDataSourceVariableExpression)MALLOC(sizeof(struct _SPTGDataSourceVariableExpression));
	n->_print = (_PTGProc)_PrPTGDataSourceVariableExpression;
	n->p1 = p1;
	return (PTGNode)n;
}

/* No Implementation of Pattern DataSourceFormalParam needed */
/* See DataSourceActualParam for implementation */


/* -------------------------------------------------------- */
/*                  Default Output Functions                */
/* -------------------------------------------------------- */


#ifdef PROTO_OK
void _PTGPrintInt(PTG_OUTPUT_FILE file, int param)
#else
void _PTGPrintInt(file, param)
	PTG_OUTPUT_FILE file; int param;
#endif
{    /* used for short and int */
	sprintf(buffer,"%d",param);
	PTG_OUTPUT_STRING(file,buffer);
}

#ifdef PROTO_OK
void _PTGPrintLong(PTG_OUTPUT_FILE file, long param)
#else
void _PTGPrintLong(file, param)
	PTG_OUTPUT_FILE file; long param;
#endif
{
	sprintf(buffer,"%ld",param);
	PTG_OUTPUT_STRING(file,buffer);
}

#ifdef PROTO_OK
void _PTGPrintDouble(PTG_OUTPUT_FILE file, double param)
#else
void _PTGPrintDouble(file, param)
	PTG_OUTPUT_FILE file; double param;
#endif
{    /* used for float and double */
	sprintf(buffer,"%g",param);
	PTG_OUTPUT_STRING(file,buffer);
}

#ifdef PROTO_OK
void _PTGPrintChar(PTG_OUTPUT_FILE file, char param)
#else
void _PTGPrintChar(file, param)
	PTG_OUTPUT_FILE file; char param;
#endif
{
	buffer[0] = param;
	buffer[1] = 0;
	PTG_OUTPUT_STRING(file,buffer);
}

