#ifndef _PTGGEN_H
#define _PTGGEN_H

#include <stdio.h>
#include "eliproto.h"

/* Include-files specified in .ptg.phi-files */

#include "ptg.h"
/* Definition of Output-Macros */

#if !defined(PTG_OUTPUT_FILE)
#    define PTG_OUTPUT_FILE FILE *
#    define PTG_OUTPUT_DEFAULT 1
#else
#    define PTG_OUTPUT_DEFAULT 0
#endif
#if !defined(PTG_OUTPUT_STRING)
#    define PTG_OUTPUT_STRING(file,param) fputs(param, file)
#    if !defined(PTG_OUTPUT_INT)
#            define PTG_OUTPUT_INT(file,param)    fprintf(file,"%d",param)
#    endif
#    if !defined(PTG_OUTPUT_LONG)
#            define PTG_OUTPUT_LONG(file,param)   fprintf(file,"%ld",param)
#    endif
#    if !defined(PTG_OUTPUT_SHORT)
#            define PTG_OUTPUT_SHORT(file,param)  fprintf(file,"%d",(int) param)
#    endif
#    if !defined(PTG_OUTPUT_CHAR)
#            define PTG_OUTPUT_CHAR(file,param)   fputc(param, file)
#    endif
#    if !defined(PTG_OUTPUT_FLOAT)
#            define PTG_OUTPUT_FLOAT(file,param)  fprintf(file,"%g",(double)param)
#    endif
#    if !defined(PTG_OUTPUT_DOUBLE)
#            define PTG_OUTPUT_DOUBLE(file,param) fprintf(file,"%g",param)
#    endif

#else
     /* PTG_OUTPUT_STRING defined */
     extern void _PTGPrintInt ELI_ARG((PTG_OUTPUT_FILE,int));
     extern void _PTGPrintLong ELI_ARG((PTG_OUTPUT_FILE,long));
     extern void _PTGPrintChar ELI_ARG((PTG_OUTPUT_FILE,char));
     extern void _PTGPrintDouble ELI_ARG((PTG_OUTPUT_FILE,double));
#    if !defined(PTG_OUTPUT_INT)
#            define PTG_OUTPUT_INT(file,param)    _PTGPrintInt(file,param)
#    endif
#    if !defined(PTG_OUTPUT_LONG)
#            define PTG_OUTPUT_LONG(file,param)   _PTGPrintLong(file,param)
#    endif
#    if !defined(PTG_OUTPUT_SHORT)
#            define PTG_OUTPUT_SHORT(file,param)  _PTGPrintInt(file,(int)param)
#    endif
#    if !defined(PTG_OUTPUT_CHAR)
#            define PTG_OUTPUT_CHAR(file,param)   _PTGPrintChar(file,param)
#    endif
#    if !defined(PTG_OUTPUT_FLOAT)
#            define PTG_OUTPUT_FLOAT(file,param)  _PTGPrintDouble(file,(double)param)
#    endif
#    if !defined(PTG_OUTPUT_DOUBLE)
#            define PTG_OUTPUT_DOUBLE(file,param) _PTGPrintDouble(file,param)
#    endif
#endif

/* Define PTGNode Type */
#if defined(__cplusplus)
struct _SPTG0;
typedef void (* _PTGProc)(struct _SPTG0 *);
#else
typedef void (* _PTGProc)();
#endif

typedef struct _SPTG0
{
  _PTGProc _print;
} * _PPTG0;
typedef _PPTG0 PTGNode;                      /* the only exported type */

/* predefined static PTGNULL-Node */

extern struct _SPTG0 _PTGNULL;
#define PTGNULL (& _PTGNULL)
#define PTGNull() (& _PTGNULL)

/* output functions */

#if PTG_OUTPUT_DEFAULT
extern PTGNode PTGOut ELI_ARG((PTGNode root));
extern PTGNode PTGOutFile ELI_ARG((char *filename, PTGNode root));
extern PTGNode PTGOutFPtr ELI_ARG((FILE *output, PTGNode root));
#endif
extern PTGNode PTGProcess ELI_ARG((PTG_OUTPUT_FILE file, PTGNode root));

/* Memory Management functions. */

extern void PTGFree ELI_ARG((void));

/* functions for making PTG nodes */

PTGNode PTGAddNL ELI_ARG((PTGNode p1));
PTGNode PTGId ELI_ARG((int p1));
PTGNode PTGAsIs ELI_ARG((char * p1));
PTGNode PTGNumb ELI_ARG((int p1));
PTGNode PTGCString ELI_ARG((char * p1));
PTGNode PTGCChar ELI_ARG((int p1));
PTGNode PTGPString ELI_ARG((char * p1));
PTGNode PTGSeq ELI_ARG((PTGNode p1, PTGNode p2));
PTGNode PTGCommaSeq ELI_ARG((PTGNode p1, PTGNode p2));
#define PTGEol PTGAddNL
PTGNode PTGWebApplication ELI_ARG((void));
PTGNode PTGWebPage ELI_ARG((PTGNode p1, PTGNode p2, PTGNode p3, PTGNode p4, PTGNode p5, PTGNode p6));
PTGNode PTGWebPageProtector ELI_ARG((PTGNode p1, PTGNode p2, PTGNode p3));
PTGNode PTGWebPageHeader ELI_ARG((void));
PTGNode PTGWebPageBody ELI_ARG((PTGNode p1, PTGNode p2));
PTGNode PTGWebPageBodyProtected ELI_ARG((PTGNode p1, PTGNode p2));
PTGNode PTGFrame ELI_ARG((PTGNode p1, PTGNode p2));
PTGNode PTGFrameRef ELI_ARG((PTGNode p1));
PTGNode PTGWebAction ELI_ARG((PTGNode p1, PTGNode p2, PTGNode p3));
PTGNode PTGPHPOperation ELI_ARG((PTGNode p1, PTGNode p2));
PTGNode PTGDataBaseOperation ELI_ARG((PTGNode p1, PTGNode p2, PTGNode p3));
PTGNode PTGRedirect ELI_ARG((PTGNode p1));
PTGNode PTGInclude ELI_ARG((PTGNode p1));
PTGNode PTGActionHandler ELI_ARG((PTGNode p1));
PTGNode PTGActionHandler2 ELI_ARG((PTGNode p1));
PTGNode PTGFormular ELI_ARG((PTGNode p1, PTGNode p2, PTGNode p3, PTGNode p4, PTGNode p5, PTGNode p6, PTGNode p7));
PTGNode PTGFormularCondition ELI_ARG((PTGNode p1, PTGNode p2));
PTGNode PTGFormularConditionDefinition ELI_ARG((PTGNode p1, PTGNode p2, PTGNode p3));
PTGNode PTGFieldName ELI_ARG((PTGNode p1));
PTGNode PTGFieldValue ELI_ARG((PTGNode p1));
PTGNode PTGFormularCheckboxGroupElement ELI_ARG((PTGNode p1));
#define PTGFormularRadioGroupElement PTGFormularCheckboxGroupElement
PTGNode PTGFormularTextElement ELI_ARG((PTGNode p1, PTGNode p2, PTGNode p3, PTGNode p4));
PTGNode PTGFormularTextAreaElement ELI_ARG((PTGNode p1, PTGNode p2, PTGNode p3, PTGNode p4));
PTGNode PTGFormularPasswordElement ELI_ARG((PTGNode p1, PTGNode p2, PTGNode p3, PTGNode p4));
PTGNode PTGFormularCheckboxElement ELI_ARG((PTGNode p1, PTGNode p2, PTGNode p3));
PTGNode PTGFormularRadioElement ELI_ARG((PTGNode p1, PTGNode p2, PTGNode p3));
PTGNode PTGFormularSelectElement ELI_ARG((PTGNode p1, PTGNode p2, PTGNode p3, PTGNode p4, PTGNode p5, PTGNode p6));
PTGNode PTGFormularPHPScriptSelectOption ELI_ARG((PTGNode p1));
PTGNode PTGFormularConstantSelectOption ELI_ARG((PTGNode p1));
PTGNode PTGFormularDataSourceSelectOption ELI_ARG((PTGNode p1, PTGNode p2, PTGNode p3));
PTGNode PTGImage ELI_ARG((PTGNode p1, PTGNode p2, PTGNode p3, PTGNode p4, PTGNode p5, PTGNode p6));
PTGNode PTGImageFragment ELI_ARG((PTGNode p1, PTGNode p2, PTGNode p3, PTGNode p4));
PTGNode PTGParagraph ELI_ARG((PTGNode p1, PTGNode p2));
#define PTGTextFragment PTGParagraph
PTGNode PTGHeading ELI_ARG((PTGNode p1, PTGNode p2, PTGNode p3, PTGNode p4));
PTGNode PTGHrSeq ELI_ARG((PTGNode p1, PTGNode p2));
#define PTGMetaInformation PTGFormularRadioGroupElement
PTGNode PTGMetaInformationElement ELI_ARG((PTGNode p1, PTGNode p2));
#define PTGStyleSheet PTGMetaInformation
PTGNode PTGStyleSheetBlock ELI_ARG((PTGNode p1, PTGNode p2));
PTGNode PTGStyleSheetPair ELI_ARG((PTGNode p1, PTGNode p2, PTGNode p3));
PTGNode PTGNewLine ELI_ARG((PTGNode p1, PTGNode p2));
PTGNode PTGRef ELI_ARG((PTGNode p1, PTGNode p2));
PTGNode PTGTable ELI_ARG((PTGNode p1, PTGNode p2));
PTGNode PTGTableRow ELI_ARG((PTGNode p1));
PTGNode PTGTableCell ELI_ARG((PTGNode p1));
PTGNode PTGOutputElement ELI_ARG((PTGNode p1));
PTGNode PTGDataSourceActualParam ELI_ARG((PTGNode p1));
PTGNode PTGListOutputElement ELI_ARG((PTGNode p1, PTGNode p2, PTGNode p3, PTGNode p4));
PTGNode PTGListOutputElementWithLimitClause ELI_ARG((PTGNode p1, PTGNode p2, PTGNode p3, PTGNode p4, PTGNode p5, PTGNode p6));
PTGNode PTGWebNodeActualParam ELI_ARG((PTGNode p1, PTGNode p2));
#define PTGWebNodeVariableExpression PTGFieldValue
PTGNode PTGWebNodeVariableExpression2 ELI_ARG((PTGNode p1));
PTGNode PTGNavigationBar ELI_ARG((PTGNode p1, PTGNode p2, PTGNode p3));
PTGNode PTGNavigationBarElement ELI_ARG((PTGNode p1, char * p2, char * p3, PTGNode p4));
PTGNode PTGPHPGlobals ELI_ARG((PTGNode p1));
PTGNode PTGPHPArray ELI_ARG((PTGNode p1, PTGNode p2));
PTGNode PTGApostroph ELI_ARG((PTGNode p1));
#define PTGDataBase() PTGNULL
PTGNode PTGDBColumn ELI_ARG((PTGNode p1, PTGNode p2));
PTGNode PTGDataSourceDB ELI_ARG((PTGNode p1, PTGNode p2));
PTGNode PTGDBFrame ELI_ARG((PTGNode p1, PTGNode p2, PTGNode p3, PTGNode p4, PTGNode p5, PTGNode p6, PTGNode p7));
PTGNode PTGDBConnect ELI_ARG((PTGNode p1, PTGNode p2, PTGNode p3, PTGNode p4, PTGNode p5, PTGNode p6));
PTGNode PTGBatchSeparator ELI_ARG((PTGNode p1, PTGNode p2));
#define PTGDataSourceFilter PTGAsIs
PTGNode PTGDataSourceDBFunction ELI_ARG((char * p1));
PTGNode PTGDataSourceUpdate ELI_ARG((PTGNode p1, PTGNode p2, char * p3));
#define PTGDataSourceInsert PTGDataSourceUpdate
#define PTGDataSourceProjectionFilter PTGDataSourceInsert
#define PTGLimitDataSource PTGDataSourceProjectionFilter
#define PTGDataSourceDelete PTGLimitDataSource
PTGNode PTGDataSourceConstantExpression ELI_ARG((PTGNode p1));
PTGNode PTGDataSourceVariableExpression ELI_ARG((PTGNode p1));
#define PTGDataSourceFormalParam PTGDataSourceActualParam

/* prototypes for used function call insertions */

void PtgOutId ELI_ARG((PTG_OUTPUT_FILE, int));
void CPtgOutstr ELI_ARG((PTG_OUTPUT_FILE, char *));
void CPtgOutchar ELI_ARG((PTG_OUTPUT_FILE, int));
void PPtgOutstr ELI_ARG((PTG_OUTPUT_FILE, char *));
void IndentNewLine ELI_ARG((PTG_OUTPUT_FILE));
void IndentDecr ELI_ARG((PTG_OUTPUT_FILE));
void IndentIncr ELI_ARG((PTG_OUTPUT_FILE));

#ifdef MONITOR
/* Monitoring support for structured values */
#define DAPTO_RESULTPTGNode(n) DAPTO_RESULT_PTR(n)
#define DAPTO_ARGPTGNode(n) DAPTO_ARG_PTR(n, PTGNode)
#endif

#endif

