
#include "vlDraw.h"

///out-file: vlDraw.h


///public:
void vlDisableSelectionBorder(TreeContext treeContext)
{
  VLList l;
  l.append("::vlDraw::disableSelectionBorder");
  l.append(treeContext);
  l.eval();
}


///public:
VLInt vlCreateObject(TreeContext treeContext, VLString kind, VLPointArray coords, VLList normalAttributes)
{
  return vlCreateObject(treeContext, kind, coords, normalAttributes, VLList());
}


///public:
VLInt vlCreateObject(TreeContext treeContext, VLString kind, VLPointArray coords, VLList normalAttributes, VLList selectAttributes)
{
  VLList l;
  l.append("::vlDraw::createObject");
  l.append(treeContext);
  l.append(kind);
  l.append(coords);
  l.append(normalAttributes);
  l.append(selectAttributes);
  return VLInt(l.eval());
}




///public:
VLPointArray vlCreateLine(TreeContext treeContext, VLRegion fromRegion, VLRegion toRegion, VLList attributes, VLPoint zeroPos, TreeContext anchorPointsContext)
{
  VLPointArray points;
  VLPointArray anchorPoints = (anchorPointsContext.isNull() ? VLList() : vlGetValue(anchorPointsContext));

  if (fromRegion.equals(toRegion)) {
    int x = toRegion.getPosSW().getX() + toRegion.getSize().getX()/2;
    int y = toRegion.getPosSW().getY();
    points.append(x-10,y);
    points.append(x-20,y+20);
    points.append(x+20,y+20);
    points.append(x+10,y);
    anchorPointsContext = VLObjectPointer();
  }
  else {
    VLPoint fromAnchor = fromRegion.getCenter();
    VLPoint toAnchor = toRegion.getCenter();
    VLPoint fromDirectionPoint = (anchorPoints.getNumbPoints()==0 ? toAnchor : anchorPoints.getFirst());
    VLPoint toDirectionPoint = (anchorPoints.getNumbPoints()==0 ? fromAnchor : anchorPoints.getLast());
    VLPoint fromPos = fromRegion.getLineIntersectionPoint(fromDirectionPoint.sub(fromAnchor));
    VLPoint toPos = toRegion.getLineIntersectionPoint(toDirectionPoint.sub(toAnchor));
    points.append(fromPos);
    for(int i=0; i<anchorPoints.getNumbPoints(); i++) {
      points.append(anchorPoints.getPoint(i).add(zeroPos));
    }
    points.append(toPos);
  }
  vlCreateObject(treeContext, "line", points, attributes, vlList("-width","2", "-fill",vlGetCfg("selectionColor")));

  if(!anchorPointsContext.isNull()) {
    // create intermediate repositions
    for(int i=1; i<points.getNumbPoints()-1; i++) {
      VLList onDoubleClick = vlList("vlDraw::onDoubleClick_removeIntermediate", anchorPointsContext, VLInt(i-1));
      vlCreateAttrReposition(anchorPointsContext, points.getPoint(i).getSquare(3), 
        vlList("vlDraw::attrRepositionFkt_adjustIntermediate", zeroPos, anchorPointsContext, VLInt(i-1)),
        vlList("-onDoubleClick", onDoubleClick));
    }

    // create handles for insertion of new points
    for(int i=0; i<points.getNumbPoints()-1; i++) {
      VLPoint intermediate = VLRegionFromPoints(points.getPoint(i), points.getPoint(i+1)).getCenter();
      vlCreateAttrReposition(anchorPointsContext, intermediate.getSquare(2), 
        vlList("vlDraw::attrRepositionFkt_addIntermediate", zeroPos, anchorPointsContext, VLInt(i)),
        vlList());
    }    
  }

  return points;
}


///public:
void vlCreateDragHandle(TreeContext treeContext, VLRegion region)
{
  VLList l;
  l.append("::vlDraw::createDragHandle");
  l.append(treeContext);
  l.append(VLPointArray(region));
  l.eval();  
}

///public:
void vlCreateInsertionLinePS(TreeContext treeContext, VLPoint pos, VLPoint size)
{
  VLList l;
  l.append("::vlDraw::createInsertionLine");
  l.append(treeContext);
  l.append(VLRegion(pos, size));
  l.eval();
}

///public:
void vlCreateInsertionPoint(TreeContext treeContext, VLRegion pos)
{
  VLList l;
  l.append("::vlDraw::createInsertionPoint");
  l.append(treeContext);
  l.append(pos);
  l.eval();
}

///public:
void vlCreateInsertionLine(TreeContext treeContext, VLPoint from, VLPoint to)
{
  VLList l;
  l.append("::vlDraw::createInsertionLine");
  l.append(treeContext);
  l.append(VLPointArray(from, to));
  l.eval();
}


///public:
void vlCreateObjectReposition(TreeContext treeContext, VLList repositionFkt)
{
  VLList l;
  l.append("::vlDraw::createObjectReposition");
  l.append(treeContext);
  l.append(repositionFkt);
  l.eval();
}


///public:
void vlCreateInsertionRegion(TreeContext treeContext, VLPointArray coords, VLList setPositionFkt, VLPoint defaultCursorSize)
{
  VLList l;
  l.append("::vlDraw::createInsertionRegion");
  l.append(treeContext);
  l.append(coords);
  l.append(setPositionFkt);
  l.append(defaultCursorSize);
  l.eval();
}


///public:
void vlCreateResizer(TreeContext sizeAttr, VLRegion coords)
{
  vlCreateResizer(sizeAttr, coords, vlList(VLString("vlDraw::setSizeFkt_std"), coords.getPos(), sizeAttr));
}


///public:
void vlCreateResizer(TreeContext sizeAttr, VLRegion coords, VLList setSizeFkt)
{
  VLList l;
  if (!sizeAttr.isNull()) {
    l.append("::vlDraw::createResizer");
    l.append(sizeAttr);
    l.append(coords);
    l.append(setSizeFkt);
    l.eval();
  }
}


///public:
void vlCreateAttrReposition(TreeContext posAttr, VLPointArray coords, VLList attrRepositionFkt, VLList options)
{
  VLList l;
  l.append("::vlDraw::createAttrReposition");
  l.append(posAttr);
  l.append(coords);
  l.append(attrRepositionFkt);
  l.append(options);
  l.eval();
}


///public:
VLPoint vlMeasure(VLString text, VLString font)
{
  VLList l;
  l.append("::vlDraw::measure");
  l.append(text);
  l.append(font);
  return l.eval();
}


///public:
VLString vlGetFont(VLString fontDescr)
{
  VLList l;
  l.append("::font::get");
  l.append(fontDescr);
  return l.eval(); 
}


///public:
VLPointArray vlMeasureImage(VLString id) 
{
  VLList l;
  l.append("::vlDraw::measureImage");
  l.append(id);
  return l.eval();
}


///public:
VLPointArray vlGetValidImage(VLString name)
{
  VLList l;
  l.append("::vlDraw::getValidImage");
  l.append(name);
  return l.eval();
}


///public:
VLValue vlGetCfg(VLString key)
{
  VLList l;
  l.append("::vlDraw::getCfg");
  l.append(key);
  return l.eval();
}


///public:
VLString vlClipText(VLString text, VLString font, VLPoint maxSize) 
{

  if (vlMeasure(text,font).getY() > maxSize.getY()) {
        return VLString();
  }

  if (vlMeasure(text,font).getX() <= maxSize.getX()) {
        return text;
  }
  else {
        VLString result;
        int len = text.getLength()-1;
        do {
           if (len<0) return VLString();
           result = VLString(text.getSubstring(0,len), "..");
           len--;
        } while(vlMeasure(result,font).getX() > maxSize.getX());
        return result;
  }
}


/////////////////////////////////////////////////////////////////////// pretty printing


///public:
VLList vlCreateKeyword(TreeContext treeContext, VLList coords, VLString text)
{
  VLList l;
  l.append("::vlDraw::createKeyword");
  l.append(treeContext);
  l.append(coords);
  l.append(text);
  return VLList(l.eval());
}

///public:
VLList vlCreateRefText(TreeContext treeContext, VLList coords, VLString text, VLList options)
{
  VLList l;
  l.append("::vlDraw::createRefText");
  l.append(treeContext);
  l.append(coords);
  l.append(text);
  l.append(options);
  return VLList(l.eval());
}

///public:
VLList vlCreateText(TreeContext treeContext, VLList coords, VLString text, VLList options)
{
  VLList l;
  l.append("::vlDraw::createText");
  l.append(treeContext);
  l.append(coords);
  l.append(text);
  l.append(options);
  return VLList(l.eval());
}

///public:
VLList vlCreateSeparators(TreeContext treeContext, VLList coords, VLList separators)
{
   VLList l;
  l.append("::vlDraw::createSeparators");
  l.append(treeContext);
  l.append(coords);
  l.append(separators);
  return VLList(l.eval());
}


///public:
VLRegion vlGetTextRegion(VLList coords1, VLList coords2)
{
   VLList l;
  l.append("::vlDraw::getTextRegion");
  l.append(coords1);
  l.append(coords2);
  return VLRegion(l.eval());
}


