
#package require prof2


namespace eval ::main {
    set devilVersion "0.12.1+"
    set initProc ""
    set appHome [file dirname [file join [pwd] $::argv0]]
}


proc main::loadAll {} {
    # Load all library files

    foreach file [glob "$main::appHome/vlEnv/*.tcl"] {
        namespace eval :: [list source $file]
    }
    foreach file [glob "$main::appHome/otherTcl/*.tcl"] {
        namespace eval :: [list source $file]
    }

    # Load images
    foreach file [glob "$main::appHome/img/*.gif"] {
        set name img::[file tail [file rootname $file]]
        image create photo $name -file $file
    }

    # Load MDI
    namespace eval :: [list source "$main::appHome/mdi/mdi.tcl"]
}


proc main::getValidImage {name} {
    if [catch {image type $name}] {
        log::log error "Unknown image: '$name'"
        return "img::default"
    } else {
        return $name
    }
}


proc main::error {msg} {
    puts $msg
    exit 1
}


proc main::getParam {key} {
    return $::main::param($key)
}

proc main::showCommandLineHelp {} {
    puts "Syntax: $cfg::commandName \[options\] \[--\] \[file1 file2 ...\]"
    puts "        $cfg::commandName -h"
    puts ""
    puts "Options:"
    puts "  -debug 1                   Activates debug mode."
    puts "  -process <path>            Process the given file."
    puts "  -processor <processorName> Use the given processor to process a file."
    puts "                             <processorName> may also be 'EXPORT_HTML'."
    puts "  -dest <path>               Store the result of file processing at given location."
    puts ""
}

proc main::readParams {} {

    # default values
    set ::main::param(files) ""
    set ::main::param(debug) 0
    set ::main::param(process) ""
    set ::main::param(processor) ""
    set ::main::param(dest) ""
    
    set state optionOrFile

    set eolMsg ""
    foreach param $::argv {
        set eolMsg ""
        switch $state {
            optionOrFile {
                if [equal $param "--"] {
                    set state file
                } elseif [equal $param "-h"] {
                    showCommandLineHelp
                    exit
                } elseif [regexp "^-(.*)" $param dummy option] {
                    if ![elementOf $option "debug process processor dest"] {
                        return "Unknown option $option"
                    }
                    set eolMsg "Option value missing"
                    set state optionValue
                } else {
                    lappend ::main::param(files) $param
                    set state file
                }
            }
            optionValue {
                set ::main::param($option) $param
                set state optionOrFile
            }
            file {
                lappend ::main::param(files) $param
                set state file                
            }
        }
    }
    if { $::main::param(debug) == "0" } {
        setDebugOff
    }

    return $eolMsg
}


proc main::autoProcess_helper {fileId} {
    if [file::startProcessor $fileId [getParam processor]] {
        exit 0
    }
}

proc main::autoProcess {fileId} {
    after idle [list main::autoProcess_helper $fileId]
    # BOGUS Komischer effekt: ohne diese Indirektion
    # oeffnet sich im Fehlerfall die Oberflaeche nicht.
}


proc main::processParams {} {
    foreach option "debug process processor dest files" {
        log::log options "$option: [getParam $option]"
    }
    if ![empty [getParam process]] {
        set fileId [file::openNoWindow [getParam process]]
        file::afterUpdate $fileId [list main::autoProcess $fileId]
        return
    } else {
        set log::mdiOutput 1
        update
    }
    foreach file [getParam files] {
        file::openWithWindow $file
    }
    set log::mdiOutput 1
}


proc main::main {} {
    loadAll

    set msg [readParams]
    if ![empty $msg] {
        puts $msg
        puts "Enter '$cfg::commandName -h' for help"
        exit
    }
        
    MDIFrame::Setup

    # Call init procs
    foreach proc $::main::initProc {
       namespace eval :: $proc
    }
    processParams
}


main::main

