
# Modul: abstractView
#
# Schnittstelle fr Sichten.
# Sicht-Implementierungen sollten von dieser Klasse erben.


namespace eval ::abstractView {} {}

proc abstractView::new {root args} {
    handleArgs [list -size $cfg::defaultClientWindowSize]
    set id [oop::new]
    oop::addClass $id "abstractView"
    oop::data $id
    set data(root) $root
    set data(file) [file::getFileOfNode $data(root)]
    set data(client) [::MDIFrame::New_Window "Dummy" $id -width [lindex $arg(size) 0] -height [lindex $arg(size) 1]]
    set data(toplevel) [::MDIFrame::GetUserpath $data(client)]
    #set data(toplevel) [toplevel .top[newint]]
    # xxx

    set data(viewType) "Abstract View"
    file::addView $data(file) $id
    return $id
}

proc abstractView::getRoot {id} {
    oop::data $id
    return $data(root)
}

proc abstractView::getFile {id} {
    oop::data $id
    return $data(file)
}

proc abstractView::getClient {id} {
    oop::data $id
    return $data(client)
}

proc abstractView::makeVisible {id} {
    oop::data $id
    MDIFrame::MakeVisible $data(client)
}

proc abstractView::close {id} {
    oop::data $id
    MDIFrame::CloseClient $data(client)
}

proc abstractView::delete {id} {
    file::removeView [getFile $id] $id
    oop::delete $id
}

proc abstractView::update {id} {
    oop::call updateTitle $id
}

proc abstractView::updateTitle {id} {
    oop::data $id

    set filename [file::getFilename [getFile $id]]
    if [empty $filename] {
        set filename "Unnamed"
    } else {
        set filename [file tail $filename]
    }
    if [file::getModified [getFile $id]] {
        set modified "*"
    } else {
        set modified ""
    }
    MDIFrame::SetTitle [getClient $id] "$data(viewType) (${modified}$filename)"
}

proc abstractView::wincmd_startProcessor {id processor} {
    file::startProcessor [getFile $id] $processor
}

proc abstractView::wincmd_undo {id} {
    file::undo [getFile $id]
}

proc abstractView::wincmd_save {id} {
    file::interactiveSave [getFile $id]
}

proc abstractView::wincmd_saveAs {id} {
    file::interactiveSaveAs [getFile $id]
}

proc abstractView::wincmd_exportHtml {id} {
    file::exportHtmlInteractive [getFile $id]
}

proc abstractView::wincmd_exportEps {id} {
    file::exportEpsInteractive [getFile $id]
}

proc abstractView::wincmd_fileInfo {id} {
    file::showInfoDialog [getFile $id]
}

proc abstractView::wincmd_showAbstree {id} {
    file::openAbstractView [getFile $id]
}

proc abstractView::wincmd_showMainView {id} {
    file::openMainView [getFile $id]
}

proc abstractView::wincmd_showErrWindow {id} {
    file::openErrorView [getFile $id]
}

proc abstractView::wincmd_findRegexp {id} {
    file::findRegexp [getFile $id]
}

proc abstractView::wincmd_closeFile {id} {
    file::tryClose [getFile $id]
}

proc abstractView::isReusable {id node type} {
    oop::data $id
    return [expr [equal $node $data(root)] && [equal $type $data(viewType)]]
}

proc abstractView::beforeDeletion {id node} {
    oop::data $id
    if [c::isAncestorOf $node $data(root)] {
        MDIFrame::CloseClient $data(client)
    }
}
