# ----------------------------------------------------------------------
# Packet zur Implementierung von Tooltips
# ----------------------------------------------------------------------
# $Id: balloon.tcl,v 1.1.1.1 2004/02/18 15:33:15 cschmidt Exp $

namespace eval ::balloon {
}

# public init
#
# Initialisiert dieses Packet

lappend "::main::initProc" "::balloon::init"

proc ::balloon::init {} {
    global bhInfo

    set bhInfo(timeout) 800
    option add *Balloonhelp*background #ffffdd widgetDefault
    option add *Balloonhelp*foreground black widgetDefault
    option add *Balloonhelp.info.wrapLength 3i widgetDefault
    option add *Balloonhelp.info.justify left widgetDefault
    option add *Balloonhelp.info.font \
	    -*-lucida-medium-r-normal-sans-*-120-* widgetDefault
    toplevel .balloonhelp -class Balloonhelp \
	    -background black -borderwidth 1 -relief flat
    label .balloonhelp.info
    pack .balloonhelp.info -side left -fill y
    wm overrideredirect .balloonhelp 1
    wm withdraw .balloonhelp

    set bhInfo(active) 1
    bind . <Destroy> {::balloon::exit}
}

proc ::balloon::exit {} {
    return
    ::balloon::setActive 0
    destroy .balloonhelp
}

# public defineMessage
#
# Definiert den Hilfetext fuer ein Control

proc ::balloon::defineMessage { win mesg } {
    global bhInfo

    if { [string length $mesg] == 0 } return
    set bhInfo($win) $mesg
    bind $win <Enter> {::balloon::Pending %W}
    bind $win <Button> {::balloon::hideMessage}
    bind $win <Leave> {::balloon::hideMessage}
}

proc ::balloon::CanvasFor { win tag mesg } {
    global bhInfo
    
    set bhInfo($win,$tag) $mesg
    $win bind $tag <Enter> [list ::balloon::CanvasPending %W $tag]
    $win bind $tag <Leave> {::balloon::hideMessage}
}

proc ::balloon::Pending { win } {
    global bhInfo
    
    ::balloon::hideMessage
    set bhInfo(pending) [after $bhInfo(timeout) [list ::balloon::Show $win]]
}

proc ::balloon::CanvasPending { win tag } {
    global bhInfo

    ::balloon::hideMessage
    set bhInfo(pending) [after 1500 [list ::balloon::CanvasShow $win $tag]]
}

# public hideMessage
#
# Entfernt eine Balloon-Help, falls sie gerade sichtbar ist.

proc ::balloon::hideMessage { } {
    global bhInfo
    if {[info exists bhInfo(pending)]} {
	after cancel $bhInfo(pending)
	unset bhInfo(pending)
    } 
    wm withdraw .balloonhelp
}

proc ::balloon::Show { win } {
    global bhInfo
    global tcl_platform

    if {$bhInfo(active)} {
	.balloonhelp.info configure -text $bhInfo($win)
##	set x [expr [winfo rootx $win]+10]
##	set y [expr [winfo rooty $win]+[winfo height $win]]
	set x [expr [winfo pointerx $win] + 10]
	set y [expr [winfo pointery $win] + 10]

# Windows must have wm geometry after wm deiconify, but it looks ugly,
# because the window flashes shortly on it's old position.
	if {$tcl_platform(platform) == "windows"} {
	    wm deiconify .balloonhelp
	    raise .balloonhelp
	    wm geometry .balloonhelp +$x+$y
	} else {
	    wm geometry .balloonhelp +$x+$y
	    wm deiconify .balloonhelp
	    raise .balloonhelp
	}

    }
    unset bhInfo(pending)
}

proc ::balloon::CanvasShow { win tag } {
    global bhInfo
    global tcl_platform

    if {$bhInfo(active)} {
	.balloonhelp.info configure -text $bhInfo($win,$tag)
	set bbox [$win bbox $tag]
# winfo rootx and winfo rooty do not work properly under Windows.
# use winfo pointerx and winfo pointery instead to position balloonhelp.
# 	set dx [expr int([$win cget -width] * [lindex [$win xview] 0])]
# 	set dy [expr int([$win cget -height] * [lindex [$win yview] 0])]
# 	set x [expr [winfo rootx $win]+[lindex $bbox 0]+10 - $dx]
# 	set y [expr [winfo rooty $win]+[lindex $bbox 3] - $dy]

	set x [expr [winfo pointerx $win] + 10]
	set y [expr [winfo pointery $win] + 10]

# Windows must have wm geometry after wm deiconify, but it looks ugly,
# because the window flashes shortly on it's old position.
	if {$tcl_platform(platform) == "windows"} {
	    wm deiconify .balloonhelp
	    raise .balloonhelp
	    wm geometry .balloonhelp +$x+$y
	} else {
	    wm geometry .balloonhelp +$x+$y
	    wm deiconify .balloonhelp
	    raise .balloonhelp
	}
    }
    unset bhInfo(pending)
}

# public setActive
#
# Setzt den Status dieses Moduls auf aktiv/nicht aktiv.

proc ::balloon::setActive {{state 1}} {
    global bhInfo

    if {$state} {
	set bhInfo(active) 1
    } else {
	::balloon::hideMessage
	set bhInfo(active) 0
    }
}
