
namespace eval basiccomp {
}

# Computes unique names for objNodes.
# Checks for uniqueness of given names.
# Supports namespaces



# --- public interface -----------------------------------------------------

namespace eval names {}


# Konfigurations-Arrays:
#
#  names::isNamespace:           Classname -> 1
#  names::prefix:                Classname -> Prefix
#  names::initialNamePrefix      Classname -> Prefix

set names::isNamespace(Root) 1



# --- public interface -----------------------------------------------------


proc basiccomp::findByFullName {root fullName args} {
    handleArgs "-var {}"
    upvar $arg(var) var

    puts "Full name: $fullName"
    set currObject $root
    set partList [splitFullName $fullName]
    puts "list: $partList"
    foreach namePart [lrange $partList 1 end] {
        puts "Name part: $namePart"
        set new [findObject $currObject $namePart] 
        puts "new: $new"
        if [empty $new] {
            puts "Not found: $namePart in $currObject"
            return 0
        }
        set currObject $new
        puts "curr: $currObject"
    }
    set var $currObject
    puts "result: $var"
    return 1
}


# PUBLIC
proc basiccomp::getFullName {objNode} {
    if [nullptr $objNode] {
        return ""
    }
    set namespaceNode [getNamespace $objNode]
    if [nullptr $namespaceNode] {
        set namespace ""
    } else {
        set namespace [getNamespaceName $namespaceNode]
    }
    return "${namespace}::[getShortName $objNode]"
}


# PUBLIC
proc basiccomp::getShortName {objNode} {
    if [nullptr $objNode] {
        return ""
    }
    objData [c::getRoot $objNode]

    set name [getPrefixedName $objNode]
    set namespace [getNamespace $objNode]
    if [info exists data(name,$namespace-$name)] {
        set list $data(name,$namespace-$name)
        if [llength $list]>1 {
            set index [lsearch $list $objNode]
            incr index
            set name "$name\[$index\]"
        }
    }
    return $name
}


# PUBLIC
proc basiccomp::getSimpleName {objNode} {
    return [getRawName $objNode]
}


proc basiccomp::getInitialNamePrefix {objNode} {
    set class [c::getClassName $objNode]
    if [info exists names::initialNamePrefix($class)] {
        return $names::initialNamePrefix($class)
    } else {
        return $class
    }
}

# PUBLIC
proc basiccomp::computeInitialName {objNode} {
    objData [c::getRoot $objNode]
    
    set namespace [getNamespace $objNode]
    set prefix [getPrefix $objNode]
    set initialNamePrefix [getInitialNamePrefix $objNode]
    set i 1

    while 1 {
        set name "$initialNamePrefix$i"
        set namespace [getNamespace $objNode]
        if ![info exists data(name,$namespace-${prefix}$name)] {
            return $name
        }
        incr i
    }
}

# PUBLIC
proc basiccomp::getErrorMessages {root messageListName} {
    upvar $messageListName messageList
    objData $root
    foreach {name objList} [array get data name,*] {
        if [llength $objList]>1 {
            foreach obj $objList {
                checkutil::appendErrorItem messageList $obj [i18n::text "Name is not unique: %s" [getFullName $obj]]
            }
        }
    }
}


# PUBLIC
proc basiccomp::getNamespace {node} {
    # returns a node, that represents the object's namespace
    # if $node is a nullptr, the nullptr is returned
    # the representation of Root's namespace is Root.

    if { [nullptr $node] || [c::isRoot $node] } {
        return $node
    }
    
    while 1 {
        set node [c::getParent $node]
        if [c::isLanguageObject $node] {
            foreach class [c::getContexts $node] {
                if [info exists names::isNamespace($class)] {
                    return $node
                }
            }
        }
        if [c::isRoot $node] {
            log::error [i18n::text "getNamespace: %s is Root object!" [c::getContext $node]]
            return 0x0
        }
    }
}


proc basiccomp::getNamespaceName {namespaceNode} {
    if [c::isRoot $namespaceNode] {
        return ""
    } else {
        return [getFullName $namespaceNode]
    }
}


# PUBLIC
proc basiccomp::getRelativeName {node base} {
    if [nullptr $node] {
        return ""
    }

    set nsNode [getNamespace $node]
    set nsBase [getNamespace $base]

    if { [equal $nsNode $nsBase] || [c::isAncestorOf $nsBase $nsNode] } {
        set cut [string length "[getNamespaceName $nsBase]::"]
    } else {
        set cut 0
    }
    return [string range [getFullName $node] $cut end]
}



# --- private functions -----------------------------------------------------



# PRIVATE
proc basiccomp::getGivenName {obj} {
    # returns given name or empty string.

    if [c::getSubElement $obj name -var nameObj] {
        return [c::getValue $nameObj]
    } else {
        return ""
    }
}


# PRIVATE
proc basiccomp::getInternalName {obj} {
    # returns internal name of obj.
    #return "obj$obj"
    return "obj[c::getPersistentId [d::getFile $obj] $obj]"
}


# PRIVATE
proc basiccomp::getRawName {obj} {
    # returns the given name or (if not present) the internal name.

    set name [getGivenName $obj]

    if [empty $name] {
        return [getInternalName $obj]
    } else {
        return $name
    }
}


# PRIVATE
proc basiccomp::getPrefix {objNode} {
    # liefert namespace-prefix des Knotens
    foreach class [c::getContexts $objNode] {
        if [info exists names::prefix($class)] {
            return "$names::prefix($class)-"
        }
    }
    return ""
}


# PRIVATE
proc basiccomp::getPrefixedName {objNode} {
    # returns name with class-specific prefix

    return [getPrefix $objNode][getRawName $objNode]
}


proc basiccomp::findObject {namespace shortName} {
    objData [c::getRoot $namespace]

    if [info exists data(name,$namespace-$shortName)] {
        return $data(name,$namespace-$shortName)
    } else {
        return ""
    }
}


# PUBLIC (called by "file" after update)
proc basiccomp::run {root} {
    objData $root
    catch {unset data}

    c::traverseTree $root isLanguageObject objNode {
        lappend data(name,[getNamespace $objNode]-[getPrefixedName $objNode]) $objNode
    }
}

proc basiccomp::splitFullName {s} {
    set result ""
    while 1 {
        set index [string first "::" $s]
        if { $index == -1 } {
            lappend result $s
            return $result
        } else {
            lappend result [string range $s 0 [expr $index-1]]
            set s [string range $s [expr $index+2] end]
        }
    }
}
        
