

namespace eval contextmenu {}

#
# Hilfsfunktionen zur Erstellung eins Kontextmenues.
#

proc contextmenu::new {parentPath} {
    set self [oop::new]
    oop::addClass $self contextmenu
    oop::data $self
    set data(menu) "$parentPath.contextmenu[newint]"
    set data(separatorPending) 0
    set data(hasEntries) 0
    set data(submenus) ""
    menu $data(menu) -tearoff 0
    return $self
}


proc contextmenu::getMenu {self} {
    oop::data $self
    return $data(menu)
}


proc contextmenu::addSeparator {self} {
    oop::data $self
    set data(separatorPending) 1
}


proc contextmenu::beforeAdd {self} {
    oop::data $self
    if { $data(separatorPending) && $data(hasEntries) } {
        $data(menu) add separator
        set data(separatorPending) 0
    }
    set data(hasEntries) 1
}


proc contextmenu::addCommand {self label command} {
    oop::data $self
    beforeAdd $self
    $data(menu) add command -label $label -command $command
}


proc contextmenu::addSubmenu {self label submenu} {
    oop::data $self
    beforeAdd $self
    $data(menu) add cascade -label $label -menu [getMenu $submenu]
    lappend data(submenus) $submenu
}


proc contextmenu::popup {self x y} {
    tk_popup [getMenu $self] $x $y
}


proc contextmenu::delete {self} {
    oop::data $self
    foreach submenu $data(submenus) {
        delete $submenu
    }
    destroy $data(menu)
    unset data
}


proc contextmenu::show {menuItems x y} {
    # Format von men: "Label Command Label Command ..."
    # wobei:
    # - Label drfen den Inhalt "---" haben. Dann wird einen Separator erzeugt
    # - Label drfen die Form "x -> y" haben. Dann ist y ein Untermeneintrag im Untermen x.

    set menuObj [new ""]
    foreach {label command} $menuItems {
        if [equal $label "---"] {
            addSeparator $menuObj
        } else {
            if [regexp "(.*) -> (.*)" $label dummy labelPart1 labelPart2] {
                if ![info exists subMenus($labelPart1)] {
                    set subMenus($labelPart1) [new [getMenu $menuObj]]
                    addSubmenu $menuObj $labelPart1 $subMenus($labelPart1)
                }
                addCommand $subMenus($labelPart1) $labelPart2 $command
            } else {
                addCommand $menuObj $label $command
            }
        }
    }
    popup $menuObj $x $y
    # XXX: popup returns immediately, so delete can't be called here!
 }

