
namespace eval exportHtml {}

proc exportHtml::quoteAnchor {s} {
    set result ""
    for {set i 0} {$i < [string length $s]} {incr i} {
        set c [string index $s $i]
        if [regexp {[a-zA-Z0-9-]} $c] {
            append result $c
        } else {
            append result "_"
        }
    }
    return $result
}

proc exportHtml::quoteFilename {s} {
    return [quoteAnchor $s]
}

proc exportHtml::saveEps {node viewName destPath} {

    set view [file::openView [file::getFileOfNode $node] $node $viewName -newView 1]
    lidoView:saveAsPostscript $view $destPath
    abstractView::close $view
}

proc exportHtml::collectViews {node destDir listVar} {
    upvar $listVar list

    if [c::isLanguageObject $node] {
        set views [treedraw::getViewNames $node]
        if [llength $views]>0 {
            set type [lindex $views 0]
            #set name "$type [basiccomp::getFullName $node]"
            set name [basiccomp::getFullName $node]
            set epsPath [file join $destDir [quoteFilename $name].eps]
            saveEps $node $type $epsPath
            lappend list $name
        }
    }
    foreach subElem [c::getSubElements $node] {
        collectViews $subElem $destDir list
    }    
}


proc exportHtml::exportHtml {file destDir} {
    export $file $destDir 1
}


proc exportHtml::exportEps {file destDir} {
    export $file $destDir 0
}


proc exportHtml::export {file destDir html} {
    # destDir must exist

    ### Collect view informations
    set viewList ""
    collectViews [file::getRoot $file] $destDir viewList

    if $html {
        ### compute index.html
        set outFile [open [file join $destDir "index.html"] w]
        puts $outFile "<html>"
        puts $outFile "<head>"
        puts $outFile "<link rel='stylesheet' href='visExp.css' type='text/css'>"
        puts $outFile "<title>[file::getFilenameNoPath $file]</title>"
        puts $outFile "</head>"
        puts $outFile ""
        puts $outFile "<body>"
        puts $outFile "<h1>[file::getFilenameNoPath $file]</h1>"

    puts $outFile "<ul>"
        foreach viewName $viewList {
            puts $outFile "<li><a href='\#[quoteAnchor $viewName]'>$viewName</a>"
        }    
        puts $outFile "</ul>"

        foreach viewName $viewList {
            puts $outFile "<h2><a name='[quoteAnchor $viewName]'>$viewName</a></h2><p><img src='[quoteFilename $viewName].png' alt='Screenshot'>"
        }    
        puts $outFile "</body>"
        close $outFile

        ### copy css
        file copy -force [file join $::main::appHome "visExp.css"] $destDir

        ### convert eps -> png
        foreach viewName $viewList {
            set from [file join $destDir [quoteFilename $viewName].eps]
            set to   [file join $destDir [quoteFilename $viewName].png]

            if [catch {exec convert -units PixelsPerInch -density 92 $from $to} errMsg] {
                append msg "Can't convert .eps to .png files!\n"
                append msg "Need external tool 'convert' (http://www.imagemagick.org) in search path."
                append msg "Error message: $errMsg"
                CDlg::Error $msg "HTML export"
                return
            }
        }
    }
}

