
namespace eval ::lidoView {}


proc lidoView::new {root viewName} {
    set id [abstractView::new $root]
    oop::addClass $id "lidoView"
    oop::data $id

    frame $data(toplevel).mainFrame
    pack $data(toplevel).mainFrame -fill both -expand 1
    
    meToolbar::create $data(toplevel).mainFrame.toolbar $viewName $data(toplevel).mainFrame.activeCanvas
    pack $data(toplevel).mainFrame.toolbar -fill y -side left

    activeCanvas::create $data(toplevel).mainFrame.activeCanvas
    pack $data(toplevel).mainFrame.activeCanvas -fill both -expand 1

    set data(toolbar) $data(toplevel).mainFrame.toolbar
    set data(activeCanvas) $data(toplevel).mainFrame.activeCanvas
    set data(viewName) $viewName
    set data(viewType) $viewName

    return $id
}

proc lidoView::update {id} {
    oop::data $id
    abstractView::update $id
    vlDraw::start $data(activeCanvas) $data(root)
    c::startProcessor $data(viewName) $data(root)
    vlDraw::end
    meToolbar::updateStatus $data(toolbar)
}

proc lidoView::wincmd_selectNext {id} {
    oop::data $id
    if [regexp "^obj-(.*)$" [activeCanvas::getSelection $data(activeCanvas)] dummy node] {
        catch {
            set new  "obj-[c::getSuccessor $node]"
            activeCanvas::trySetSelectionAndScroll $data(activeCanvas) $new
        }
    }
}

proc lidoView::wincmd_selectPrev {id} {
    oop::data $id
    if [regexp "^obj-(.*)$" [activeCanvas::getSelection $data(activeCanvas)] dummy node] {
        catch {
            set new "obj-[c::getPredecessor $node]"
            activeCanvas::trySetSelectionAndScroll $data(activeCanvas) $new
        }
    }
}

proc lidoView::wincmd_selectParent {id} {
    oop::data $id
    if [regexp "^obj-(.*)$" [activeCanvas::getSelection $data(activeCanvas)] dummy node] {
        catch {
            set new "obj-[c::getParent [c::getParent $node]]"
            activeCanvas::trySetSelectionAndScroll $data(activeCanvas) $new
        }
    }
}

proc lidoView::wincmd_selectFirstChild {id} {
    oop::data $id
    if [regexp "^obj-(.*)$" [activeCanvas::getSelection $data(activeCanvas)] dummy node] {
        catch {
            set new "obj-[c::getFirst [lindex [getNonEmptyAggSubElements $node] 0]]"
            activeCanvas::trySetSelectionAndScroll $data(activeCanvas) $new
        }
    }
}


proc lidoView::wincmd_selectNextChild {id} {
    oop::data $id
    if [regexp "^obj-(.*)$" [activeCanvas::getSelection $data(activeCanvas)] dummy node] {
        catch {
            set current [c::getParent $node]
            set list [getNonEmptyAggSubElements [c::getParent $current]]
            set index [lsearch -exact $list $current]
            set newAgg [lindex $list [expr ($index + 1) % [llength $list]]]
            set new "obj-[c::getFirst $newAgg]"
            activeCanvas::trySetSelectionAndScroll $data(activeCanvas) $new
        }
    }
}

proc lidoView::wincmd_selectAll {id} {
    oop::data $id
    puts "select all"
    activeCanvas::trySetSelectionAndScroll $data(activeCanvas) "obj-$data(root)"
}


proc lidoView::wincmd_editProperties {id} {
    oop::data $id
    if [regexp "^obj-(.*)$" [activeCanvas::getSelection $data(activeCanvas)] dummy node] {
        treedraw::editAttributes $node
    }
}


proc lidoView::wincmd_delete {id} {
    oop::data $id
    if [regexp "^obj-(.*)$" [activeCanvas::getSelection $data(activeCanvas)] dummy node] {
        edit::delete $node
    }
}


proc lidoView::wincmd_cut {id} {
    oop::data $id
    if [regexp "^obj-(.*)$" [activeCanvas::getSelection $data(activeCanvas)] dummy node] {
        edit::cutIntoClipboard $node
    }
}


proc lidoView::wincmd_copy {id} {
    oop::data $id
    if [regexp "^obj-(.*)$" [activeCanvas::getSelection $data(activeCanvas)] dummy node] {
        edit::copyIntoClipboard $node
    }
}

proc lidoView::wincmd_paste {id} {
    oop::data $id
    meToolbar::on_pasteClipboard $data(toolbar)
}


proc lidoView::wincmd_saveAsPostscript {id} {
    set types ""
    lappend types [list "Encapsulated Postscript" .eps]
    lappend types [list "All Files" "*"]
    set filename [tk_getSaveFile -filetypes $types -defaultextension ".eps"]
    if ![empty $filename] {
        saveAsPostscript $filename
    }
}


proc lidoView:saveAsPostscript {id filename} {
    oop::data $id
    activeCanvas::saveAsPostscript $data(activeCanvas) $filename    
}


proc lidoView::getNonEmptyAggSubElements {obj} {
    set list [c::getSubElements $obj]
    set result ""
    foreach x $list {
        if { [c::isAggregate $x] && [c::getFirst $x]!=[c::getEOL $x] } {
            lappend result $x
        }
    }
    return $result
}


proc lidoView::selectNode {id node} {
    oop::data $id
    while { ![activeCanvas::containsGroup $data(activeCanvas) obj-$node] } {        
        set node [c::getParent $node]
    }
    MDIFrame::MakeVisible $data(client)
    activeCanvas::setSelectionWithEffect $data(activeCanvas) "obj-$node"
}


proc lidoView::isReusable {id node type} {
    oop::data $id
    return [expr [equal $node $data(root)] && [equal $type $data(viewType)]]
}

