
namespace eval meToolbar {}


proc meToolbar::create {obj viewName activeCanvas} {
    # Creates a virtual widget with path $obj

    upvar 0 ::meToolbar::$obj data
    set data(frame) $obj
    set data(pressed) ""
    set data(activeCanvas) $activeCanvas
    set data(buttons) ""

    frame $data(frame)
    
    if [info exists ::meToolbar::buttons($viewName)] {
        foreach {label type infoText class} $::meToolbar::buttons($viewName) {
            set buttonId "button[newint]"
            lappend data(buttons) $buttonId
            set data(class,$buttonId) $class
            switch $type {
                "text" {
                    button $data(frame).$buttonId -text $label -command [list meToolbar::on_buttonPress $obj $buttonId] -state disabled
                }
                "image" {
                    button $data(frame).$buttonId -image [main::getValidImage $label] -command [list meToolbar::on_buttonPress $obj $buttonId] -state disabled
                }
                default {
                    log::log error "Invalid button type: $type"
                }
            }
            pack $data(frame).$buttonId -fill x -side top
            if { ![empty $infoText] } {
                balloon::defineMessage $data(frame).$buttonId $infoText
            }
        }
    }
    bind all <ButtonPress-1> +[list meToolbar::on_mouseDown $obj]
}


proc meToolbar::releaseButtons {obj} {
    upvar 0 ::meToolbar::$obj data
    if ![empty $data(pressed)] {
        if ![equal $data(pressed) "CLIPBOARD"] {
            $data(frame).$data(pressed) configure -relief raised
        }
        activeCanvas::returnToObjectSelection $data(activeCanvas)        
        set data(pressed) ""
    }
}


proc meToolbar::canInsert {className dropContexts} {
    set typeId [c::getTypeIdFromName $className]
    foreach dropContext $dropContexts {
        if { [c::canInsertType $dropContext $typeId] } {
            return 1
        }
    }
    return 0
}


proc meToolbar::updateStatus {obj} {
    upvar 0 ::meToolbar::$obj data
    set dropGroups [activeCanvas::getDropTargetGroups $data(activeCanvas)]

    # "inspoints" herausfiltern
    set dropContexts ""
    foreach dropGroup $dropGroups {
        if [equal [vlDraw::getType $dropGroup] "inspoint"] {
            lappend dropContexts [vlDraw::getTreeContext $dropGroup]
        }
    }

    foreach buttonId $data(buttons) {          
        if { [canInsert $data(class,$buttonId) $dropContexts] } {
            $data(frame).$buttonId configure -state normal
        } else {
            $data(frame).$buttonId configure -state disabled
        }
    }
}


proc meToolbar::updateStatus_old {obj} {
    upvar 0 ::meToolbar::$obj data
    foreach buttonId $data(buttons) {
        if [activeCanvas::canInsert $data(activeCanvas) [list vlDraw::acceptDropPredicate_type $data(class,$buttonId)]] {
            $data(frame).$buttonId configure -state normal
        } else {
            $data(frame).$buttonId configure -state disabled
        }
    }
}



proc meToolbar::on_buttonPress {obj buttonId} {
    upvar 0 ::meToolbar::$obj data
    set data(pressed) $buttonId
    $data(frame).$buttonId configure -relief sunken
    activeCanvas::startObjectInsertion $data(activeCanvas) \
        [list vlDraw::acceptDropPredicate_type $data(class,$buttonId)] [list vlDraw::onDrop_insertNew $data(class,$buttonId)]
}


proc meToolbar::on_pasteClipboard {obj} {
    upvar 0 ::meToolbar::$obj data
    set data(pressed) "CLIPBOARD"
    activeCanvas::startObjectInsertion $data(activeCanvas) \
        vlDraw::acceptDropPredicate_clipboard vlDraw::onDrop_insertClipboard
}

proc meToolbar::on_mouseDown {obj} {
    upvar 0 ::meToolbar::$obj data
    releaseButtons $obj
}
