
namespace eval messageView {}


proc messageView::new {root title messageList} {
    set id [abstractView::new $root -size "700 200"]
    oop::addClass $id "messageView"
    oop::data $id

    frame $data(toplevel).sCanvas
    pack $data(toplevel).sCanvas -fill both -expand 1

    canvas $data(toplevel).sCanvas.workspace -background white
    CDlg::mkScrollableXY $data(toplevel).sCanvas workspace

    set data(canvas) "$data(toplevel).sCanvas.workspace"
    set data(viewType) $title
    set data(messageList) $messageList
    update $id
    return $id
}

proc messageView::update {id} {
    oop::data $id
    abstractView::update $id
    $data(canvas) delete all

    set posx 10
    set posy 10
    set font [font::get "_messageView"]
    set linespace [font metrics $font -linespace]
    set pad 5

    set messageList $data(messageList)
    if [empty $messageList] {
        set messageList [list [list 0x0 white [i18n::text "No results"]]]
    }
    
    set maxWidth 10
    foreach item $messageList {
        assign "context color message" $item
        set maxWidth [max $maxWidth [font measure $font $message]]
    }

    foreach item $messageList {
        assign "context color message" $item

        set id0 [$data(canvas) create rectangle $posx $posy [expr $posx+$maxWidth] [expr $posy+$linespace] -fill $color -outline ""]
        set id1 [$data(canvas) create text $posx $posy -anchor nw -text $message -font $font]
        if ![nullptr $context] {
            $data(canvas) bind $id0 <Button-1> [list messageView::clickExec file::highlightNode $context]
            $data(canvas) bind $id1 <Button-1> [list messageView::clickExec file::highlightNode $context]
            $data(canvas) bind $id0 <Button-3> [list messageView::contextMenu $context %X %Y]
            $data(canvas) bind $id1 <Button-3> [list messageView::contextMenu $context %X %Y]

        }
        incr posy [expr $linespace+$pad]
    }

    assign "x1 y1 x2 y2" [$data(canvas) bbox all]
    $data(canvas) config -scrollregion "0 0 [expr $x2+10] [expr $y2+10]"
}

proc messageView::clickExec {args} {
    after idle $args
}

proc messageView::contextMenu {elem x y} {
    set menuList ""
    lappend menuList [i18n::text "Highlight node in tree view"] [list file::highlightNodeInTreeView $elem]
    contextmenu::show $menuList $x $y
}
