
# treedraw.tcl
#
# Vormals Methoden von treeView.tcl, die aber auch
# in anderen Sichten verwendet werden.


namespace eval ::treedraw {}



proc treedraw::showNodeInfo {objNode} {
    lappend message "Node ID: $objNode"
    lappend message "Simple name: '[basiccomp::getSimpleName $objNode]'"
    lappend message "Short name: '[basiccomp::getShortName $objNode]'"
    lappend message "Full name: '[basiccomp::getFullName $objNode]'"
    lappend message "Tree size: [c::getObjCount $objNode]"

    CDlg::Information [join $message "\n\n"] "Node Information"
}



proc treedraw::getViewNames {elem} {
    set className [c::getClassName $elem]
    if [info exists ::treedraw::viewnames($className)] {
        return $::treedraw::viewnames($className)
    } else {
        return ""
    }
}


proc treedraw::collectObjects {root listName} {
    upvar $listName list
    lappend list $root
    foreach aggObj [c::getSubElements $root] {
        foreach sub [c::getSubElements $aggObj] {
            collectObjects $sub list
        }
    }
}



proc treedraw::getVLPointerList {obj} {
    # Returns a list containing names and values of all valid references,
    # that can be stored in $obj.

    set currentValue [c::getValue $obj]
    set listContainsCurrentValue 0

    # collect objects
    set list ""
    collectObjects [c::getRoot $obj] list

    # check and add label
    set list2 ""
    foreach x $list {
        if [empty [checkutil::doValueCheck $obj $x]] {
            if [equal $x $currentValue] {
                set listContainsCurrentValue 1
            }
            set relName [basiccomp::getRelativeName $x $obj]
            switch -- [string first "::" $relName] {
                -1      {set sortPrefix a}
                0       {set sortPrefix c}
                default {set sortPrefix b}
            }
            lappend list2 [list $sortPrefix$relName $x]
        }
    }
    
    # sort
    set list2 [lsort -index 0 -dictionary $list2]

    # flatten
    set list ""
    lappend list [stringRep::nullValue2] "0x0"
    if {!$listContainsCurrentValue && ![nullptr $currentValue] } {
        lappend list "([basiccomp::getRelativeName $currentValue $obj])" $currentValue
    }
    foreach x $list2 {
        lappend list [string range [lindex $x 0] 1 end] [lindex $x 1]
    }
    return $list
}


proc treedraw::getEditor {obj} {

    # --- try to choose context based

    foreach context [c::getContexts $obj] {
        if [info exists treedraw::editor($context)] {
            return $treedraw::editor($context)
        }
        if [info exists treedraw::editorComputedby($context)] {
            return [eval $treedraw::editorComputedby($context) $obj]
        }
    }

    # --- try to choose type based

    set type [c::getTypeName $obj]

    set ed(VLBoolean) Checkbox
    set ed(VLString) Entry
    set ed(VLInt) Entry
    if [info exists ed($type)] {
        return $ed($type)
    }
    if [equal $type "VLPointer"] {
        set values [getVLPointerList $obj]
        return [list Listbox -values $values]
    }

    # --- default value
    return "Entry"
}


proc treedraw::editAttributesEx {list title} {
    set dcList ""
    foreach obj $list {
        set data($obj,value) [c::getValue $obj]
        set data($obj,label) [c::getRole $obj]
        set data($obj,editor) [getEditor $obj]
        set data($obj,check) "checkutil::doValueCheck $obj"
        if ![equal $data($obj,editor) "None"] {
            lappend dcList $obj
        }
    }
    if ![empty $dcList] {
        set result [CDlg::stdInputDialog $title $dcList data]
        if $result {
            foreach obj $list {
                edit::setValue $obj $data($obj,value)
            }
        }
    }
}


proc treedraw::editAttribute {obj} {
    editAttributesEx [list $obj] "Edit Attribute"
}

proc treedraw::orderAttributeList {list class} {
    if { ![info exists ::treedraw::attrOrder($class)] } {
        return $list
    }
    set order $::treedraw::attrOrder($class)
    puts "order: $order"
    set result ""
    foreach elem $order {
        set newList ""
        foreach id $list {
            if { [equal [c::getRole $id] $elem] } {
                lappend result $id
            } else {
                lappend newList $id
            }
        }
        set list $newList
    }
    return [concat $result $list]
}


proc treedraw::openStdAttributeDialog {obj} {
    # called by file::openAttributeDialog
    set list [lfilter [c::getSubElements $obj] {[c::isValue $x] || [c::isReference $x]}]
    set list [orderAttributeList $list [c::getClassName $obj]]
    editAttributesEx  $list "Edit [c::getClassName $obj] properties"
}    


proc treedraw::editAttributes {obj} {
    file::openAttributeDialog [file::getFileOfNode $obj] $obj 
}


proc treedraw::getSimpleName {elem} {
    if [nullptr $elem] {
        return [stringRep::nullValue]
    } else {
        return [basiccomp::getSimpleName $elem]
    }
}


proc treedraw::getShortName {elem} {
    if [nullptr $elem] {
        return [stringRep::nullValue]
    } else {
        return [basiccomp::getShortName $elem]
    }
}



proc treedraw::getFullName {elem} {
    if [nullptr $elem] {
        return [stringRep::nullValue]
    } else {
        return [basiccomp::getFullName $elem]
    }
}


proc treedraw::getValue {elem} {
    if [c::isReference $elem] {
        set result [getFullName [c::getValue $elem]]
    } else {
        set result [c::getValue $elem]
    }
    return [stringRep::quoted $result]
}



# --- context menu creation -------------------------------------------------------------------------------



proc treedraw::getSpecialContextmenu {elem menuListName} {
    upvar $menuListName menuList

    # Can be overwritten to support special operations in context menues.
    # elem can be a objNode or a valNode.
    #lappend menuList "Say hello" [list puts "Hello from node $elem!"]
}


proc treedraw::getGeneralContextmenu {elem menuListName} {
    upvar $menuListName menuList
    lappend menuList "Highlight in tree view" [list file::highlightNodeInTreeView $elem]
    getSpecialContextmenu $elem menuList
}


proc treedraw::getEditContextmenu {obj prefix listName} {
    # obj ist ein EOLObject!
    upvar $listName commands

    set parent [c::getParent $obj]
    if { [c::maxMultiplicityIsOne $parent] && [c::getSubElementCount $parent]==1 } {
        return
    }

    set typelist [c::getSubtypes [edit::getAcceptedType $obj]]

    foreach type $typelist {
        if ![c::getIsAbstract $type] {
            lappend commands "${prefix}Insert [c::getNameOfType $type]" [list edit::insertNew $obj [c::getNameOfType $type]]
        }
    }
    if [clipboard::canPaste $obj] {
        lappend commands "${prefix}Paste from Clipboard" [list clipboard::paste $obj]
    }
}


proc treedraw::getObjNodeContextmenu {elem listName} {
    upvar $listName commands

    ### Part 1: general entries
    lappend commands "Edit attributes" "treedraw::editAttributes $elem"
    lappend commands "Find usages" "file::findUsages [file::getFileOfNode $elem] $elem"
    lappend commands "Show node infos" "treedraw::showNodeInfo $elem"
    lappend commands "Export subtree" "xmlio::exportSubtreeInteractive $elem"
    getGeneralContextmenu $elem commands
    lappend commands "---" ""

    ### Part 2: open other views
    foreach viewName [getViewNames $elem] {
        lappend commands "Open $viewName" [list file::openView [file::getFileOfNode $elem] $elem $viewName]
    }
    lappend commands "---" ""

    ### Part 3: edit commands
    if [edit::canExtract $elem] {
        lappend commands "Delete" [list edit::delete $elem]
        lappend commands "Cut"    [list edit::cutIntoClipboard $elem]
        lappend commands "Mark"   [list edit::mark $elem]
        lappend commands "Copy"   [list edit::copyIntoClipboard $elem]
        getEditContextmenu [c::icBefore $elem] "Before -> " commands
        getEditContextmenu [c::icAfter  $elem] "After -> " commands
    }
    foreach sub [c::getAggregateSubElements $elem] {
        # creation commands for all SUB-Attributes
        getEditContextmenu [c::icBelow $sub] "ATTR [c::getRole $sub] -> " commands
    }
    lappend commands "---" ""
}


proc treedraw::getAttrNodeContextmenu {elem listName} {
    upvar $listName commands

    if [c::isLeaf $elem] {
        lappend commands "Edit attribute" "treedraw::editAttribute $elem"        
    }
    if [c::isAggregate $elem] {
        getEditContextmenu [c::icBelow $elem] "" commands
    }
    if [c::isReference $elem] {
        lappend commands "Highlight referenced object" [list file::highlightNodeInTreeView [c::getValue $elem]]
    }

    getGeneralContextmenu $elem commands
    lappend commands "---" ""
}


proc treedraw::contextMenu {elem x y} {
    # creates contextmenu for objNode or attrNode
    set menuList ""
    if [c::isLanguageObject $elem] {
        getObjNodeContextmenu $elem menuList
    }
    if [c::isRelationObject $elem] {
        getAttrNodeContextmenu $elem menuList
    }
    contextmenu::show $menuList $x $y
}

# ---------------------------------------------------------------------------------------------------



